# Copyright (C) 2014 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np
from phonopy.structure.atoms import PhonopyAtoms
from phonopy.structure.cells import get_supercell
from phonopy.harmonic.force_constants import distribute_force_constants
from phonopy.structure.cells import SNF3x3


def get_commensurate_points(supercell_matrix):  # wrt primitive cell
    """Commensurate q-points are returned.

    Parameters
    ----------
    supercell_matrix : array_like
        Supercell matrix with respect to primitive cell basis vectors.
        shape=(3, 3), dtype=int

    Returns
    -------
    commensurate_points : ndarray
        Commensurate points corresponding to supercell matrix.
        shape=(N, 3), dtype='double', order='C'
        where N = det(supercell_matrix)

    """

    smat = np.array(supercell_matrix, dtype=int)
    rec_primitive = PhonopyAtoms(numbers=[1],
                                 scaled_positions=[[0, 0, 0]],
                                 cell=np.diag([1, 1, 1]),
                                 pbc=True)
    rec_supercell = get_supercell(rec_primitive, smat.T)
    q_pos = rec_supercell.scaled_positions
    return np.array(np.where(q_pos > 1 - 1e-15, q_pos - 1, q_pos),
                    dtype='double', order='C')


def get_commensurate_points_in_integers(supercell_matrix):
    """Commensurate q-points in integer representation are returned.

    A set of integer representation of lattice points is transformed to
    the equivalent set of lattice points in fractional coordinates with
    respect to supercell basis vectors by
        integer_lattice_points / det(supercell_matrix)

    Parameters
    ----------
    supercell_matrix : array_like
        Supercell matrix with respect to primitive cell basis vectors.
        shape=(3, 3)
        dtype=intc

    Returns
    -------
    lattice_points : ndarray
        Integer representation of lattice points in supercell.
        shape=(N, 3)

    """
    smat = np.array(supercell_matrix, dtype=int)
    snf = SNF3x3(smat.T)
    snf.run()
    D = snf.A.diagonal()
    b, c, a = np.meshgrid(range(D[1]), range(D[2]), range(D[0]))
    lattice_points = np.dot(np.c_[a.ravel() * D[1] * D[2],
                                  b.ravel() * D[0] * D[2],
                                  c.ravel() * D[0] * D[1]], snf.Q.T)
    lattice_points = np.array(lattice_points % np.prod(D),
                              dtype='intc', order='C')
    return lattice_points


class DynmatToForceConstants(object):
    """Transforms eigensolutions to force constants

    This is the inverse transform of force constants to eigensolutions.
    Eigenvalue-eigenvector pairs or frequency-eigenvector pairs of
    dynamical matrices are transformed to force constants by this class.
    Optionally, dynamical matrices (not the DynamicalMatrix class instance,
    but the calculated matrices) can be used instead of above pairs.

    The eigensolutions have to be those calculated at commensurate q-points
    of supercell matrix. The commensurate q-points are obtained by the
    attribute ``commensurate_points``. This set of commensurate points can be
    also given by setter of it.

    One supposed usage is::

       d2f = DynmatToForceConstants(primitive, supercell)
       comm_points = d2f.commensurate_points
       ... calculated phonons at comm_points
       d2f.create_dynamical_matrices(frequencies=frequencies,
                                     eigenvectors=eigenvectors)
       d2f.run()
       fc = d2f.force_constants

    Another usage may be with eigensolutions at commensurate q-points::

       d2f = DynmatToForceConstants(primitive, supercell,
                                    commensurate_points=comm_points,
                                    eigenvalues=eigenvalues,
                                    eigenvectors=eigenvectors)
       d2f.run()
       fc = d2f.force_constants


    Attributes
    ----------
    force_constants : ndarray
        Calculated force constants. Only getter. Array shape depends on
        ``is_full_fc``.
            True : shape=(supercell_atoms, supercell_atoms, 3, 3)
            False : shape=(primitive_atoms, supercell_atoms, 3, 3)
        Default is True.
        dtype='double', order='C'.
    commensurate_points : ndarray
        Commensurate q-points of supercell matrix.
        shape=(det(supercell_matrix), 3), dtype='double', order='C'.
    dynamical_matrices : ndarray
        shape=(det(supercell_matrix), num_band, num_band),
        where num_band is 3 x number of atoms in primitive cell.
        dtype='complex128', order='C'.

    """

    def __init__(self,
                 primitive,
                 supercell,
                 frequencies=None,
                 eigenvalues=None,
                 eigenvectors=None,
                 dynamical_matrices=None,
                 commensurate_points=None,
                 is_full_fc=True):
        """

        Note
        ----
        When calculating force constants from eigensolutions,
        eigher one of frequency-eigenvector pair or eigenvalu-eigenvector
        can be used.

        Parameters
        ----------
        supercell : Supercell
            Supercell.
        primitive : Primitive
            Primitive cell
        frequencies : ndarray
            Phonon frequencies as the solution of dynamical matrices at
            commensurate q-points.
            shape=(det(supercell_matrix), num_band), dtype='double', order='C'
            where ``num_band`` is 3 x number of atoms in primitive cell.
        eigenvalues : ndarray
            Phonon eigenvalues as the solution of dynamical matrices at
            commensurate q-points.
            shape=(det(supercell_matrix), num_band), dtype='double', order='C'
            where ``num_band`` is 3 x number of atoms in primitive cell.
        eigenvectors : ndarray
            Phonon eigenvectors as the solution of dynamical matrices at
            commensurate q-points.
            shape=(det(supercell_matrix), num_band, num_band)
            dtype='complex128', order='C'.
            where ``num_band`` is 3 x number of atoms in primitive cell.
        dynamical_matrices : ndarray
            Dynamical matrices at commensurate q-points.
            shape=(det(supercell_matrix), num_band, num_band)
            dtype='complex128', order='C'.
            where ``num_band`` is 3 x number of atoms in primitive cell.
        commensurate_points : ndarray
            Commensurate q-points corresponding to supercell_matrix. The order
            has to be the same as those of frequencies or eigenvalues, and
            eigenvectors. As the default behaviour, commensurate q-points are
            generated when this is not given.
            shape=(det(supercell_matrix), 3), dtype='double', order='C'
        is_full_fc : bool
            This controls the matrix shape of calculated force constants.
            True and False give the full and compact force cosntants,
            respectively. The default is True. See more details in Attributes
            section of this class.

        """
        self._primitive = primitive
        self._supercell = supercell
        supercell_matrix = np.linalg.inv(self._primitive.primitive_matrix)
        supercell_matrix = np.rint(supercell_matrix).astype('intc')
        if commensurate_points is None:
            self._commensurate_points = get_commensurate_points(
                supercell_matrix)
        else:
            self._commensurate_points = commensurate_points
        (self._shortest_vectors,
         self._multiplicity) = primitive.get_smallest_vectors()
        self._dynmat = None
        n_s = self._supercell.get_number_of_atoms()
        n_p = self._primitive.get_number_of_atoms()
        if is_full_fc:
            fc_shape = (n_s, n_s, 3, 3)
        else:
            fc_shape = (n_p, n_s, 3, 3)
        self._fc = np.zeros(fc_shape, dtype='double', order='C')

        itemsize = self._fc.itemsize
        self._dtype_complex = ("c%d" % (itemsize * 2))

        if ((frequencies is not None or eigenvalues is not None)
            and eigenvectors is not None):
            self.create_dynamical_matrices(
                frequencies=frequencies,
                eigenvalues=eigenvalues,
                eigenvectors=eigenvectors)
        elif dynamical_matrices is not None:
            self.dynamical_matrices = dynamical_matrices

    def run(self):
        self._inverse_transformation()
        # Full fc
        # self._distribute_force_constants()

    @property
    def force_constants(self):
        return self._fc

    def get_force_constants(self):
        return self.force_constants

    @property
    def commensurate_points(self):
        return self._commensurate_points

    def get_commensurate_points(self):
        return self.commensurate_points

    @commensurate_points.setter
    def commensurate_points(self, comm_points):
        self._commensurate_points = np.array(
            comm_points, dtype='double', order='C')

    @property
    def dynamical_matrices(self):
        return self._dynmat

    def get_dynamical_matrices(self):
        return self.dynamical_matrices

    @dynamical_matrices.setter
    def dynamical_matrices(self, dynmat):
        self._dynmat = np.array(dynmat, dtype=self._dtype_complex, order='C')

    def set_dynamical_matrices(self, dynmat):
        self.dynamical_matrices = dynmat

    def create_dynamical_matrices(self,
                                  frequencies=None,
                                  eigenvalues=None,
                                  eigenvectors=None):
        dm = []
        if eigenvalues is not None:
            for eigvals, eigvecs in zip(eigenvalues, eigenvectors):
                dm.append(np.dot(np.dot(eigvecs, np.diag(eigvals)),
                                 eigvecs.T.conj()))
        else:
            for freqs, eigvecs in zip(frequencies, eigenvectors):
                eigvals = freqs ** 2 * np.sign(freqs)
                dm.append(np.dot(np.dot(eigvecs, np.diag(eigvals)),
                                 eigvecs.T.conj()))

        self.dynamical_matrices = dm

    def _inverse_transformation(self):
        try:
            import phonopy._phonopy as phonoc
            self._c_inverse_transformation()
        except ImportError:
            self._py_inverse_transformation()

        if self._fc.shape[0] == self._fc.shape[1]:
            distribute_force_constants_by_translations(self._fc,
                                                       self._primitive,
                                                       self._supercell)

    def _c_inverse_transformation(self):
        import phonopy._phonopy as phonoc

        s2p = self._primitive.s2p_map
        p2p = self._primitive.p2p_map
        s2pp = np.array([p2p[i] for i in s2p], dtype='intc')

        if self._fc.shape[0] == self._fc.shape[1]:
            fc_index_map = self._primitive.p2s_map
        else:
            fc_index_map = np.arange(self._fc.shape[0], dtype='intc')

        phonoc.transform_dynmat_to_fc(self._fc,
                                      self._dynmat.view(dtype='double'),
                                      self._commensurate_points,
                                      self._shortest_vectors,
                                      self._multiplicity,
                                      self._primitive.masses,
                                      s2pp,
                                      fc_index_map)

    def _py_inverse_transformation(self):
        s2p = self._primitive.s2p_map
        p2s = self._primitive.p2s_map
        p2p = self._primitive.p2p_map

        m = self._primitive.masses
        N = (self._supercell.get_number_of_atoms() /
             self._primitive.get_number_of_atoms())

        for p_i, s_i in enumerate(p2s):
            for s_j, p_j in enumerate([p2p[i] for i in s2p]):
                coef = np.sqrt(m[p_i] * m[p_j]) / N
                fc_elem = self._sum_q(p_i, s_j, p_j) * coef
                if self._fc.shape[0] == self._fc.shape[1]:
                    self._fc[s_i, s_j] = fc_elem
                else:
                    self._fc[p_i, s_j] = fc_elem

    def _sum_q(self, p_i, s_j, p_j):
        multi = self._multiplicity[s_j, p_i]
        pos = self._shortest_vectors[s_j, p_i, :multi]
        sum_q = np.zeros((3, 3), dtype=self._dtype_complex, order='C')
        phases = -2j * np.pi * np.dot(self._commensurate_points, pos.T)
        phase_factors = np.exp(phases).sum(axis=1) / multi
        for i, coef in enumerate(phase_factors):
            sum_q += self._dynmat[i,
                                  (p_i * 3):(p_i * 3 + 3),
                                  (p_j * 3):(p_j * 3 + 3)] * coef
        return sum_q.real


def distribute_force_constants_by_translations(fc, primitive, supercell):
    s2p = primitive.get_supercell_to_primitive_map()
    p2s = primitive.get_primitive_to_supercell_map()
    positions = supercell.get_scaled_positions()
    lattice = supercell.get_cell().T
    diff = positions - positions[p2s[0]]
    trans = np.array(diff[np.where(s2p == p2s[0])[0]],
                     dtype='double', order='C')
    rotations = np.array([np.eye(3, dtype='intc')] * len(trans),
                         dtype='intc', order='C')
    permutations = primitive.get_atomic_permutations()
    distribute_force_constants(fc,
                               p2s,
                               lattice,
                               rotations,
                               permutations)



# Copyright (C) 2016 Antti J. Karttunen (antti.j.karttunen@iki.fi)
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import numpy as np

from phonopy.file_IO import iter_collect_forces
from phonopy.interface.vasp import check_forces, get_drift_forces
from phonopy.structure.atoms import PhonopyAtoms as Atoms
from phonopy.structure.atoms import symbol_map
from phonopy.structure.cells import get_cell_parameters, get_angles
from phonopy.units import Hartree, Bohr
from phonopy.structure.symmetry import Symmetry

def parse_set_of_forces(num_atoms,
                        forces_filenames,
                        verbose=True):
    hook = 'ATOM                     X                   Y                   Z'
    force_sets = []
    for i, filename in enumerate(forces_filenames):
        if verbose:
            sys.stdout.write("%d. " % (i + 1))
        crystal_forces = iter_collect_forces(filename,
                                           num_atoms,
                                           hook,
                                           [2, 3, 4])
        if check_forces(crystal_forces, num_atoms, filename, verbose=verbose):
            is_parsed = True
            drift_force = get_drift_forces(crystal_forces,
                                           filename=filename,
                                           verbose=verbose)
            # Convert forces Hartree / Bohr ->  eV / Angstrom
            # This avoids confusion with the units. CRYSTAL uses Angstroms for
            # coordinates, but Hartree / Bohr for forces. This would lead in mixed
            # units hartree / (Angstrom * Bohr) for force constants, requiring
            # additional tweaks for unit conversions in other parts of the code
            force_sets.append(np.multiply(np.array(crystal_forces) - drift_force, Hartree / Bohr))
        else:
            is_parsed = False

    if is_parsed:
        return force_sets
    else:
        return []

def read_crystal(filename):
    f_crystal = open(filename)
    crystal_in = CrystalIn(f_crystal.readlines())
    f_crystal.close()
    tags = crystal_in.get_tags()


    cell = Atoms(cell=tags['lattice_vectors'],
                 symbols=tags['atomic_species'],
                 scaled_positions=tags['coordinates'])

    magmoms = tags['magnetic_moments']
    if magmoms is not None:
        # Print out symmetry information for magnetic cases
        # Original code from structure/symmetry.py
        symmetry = Symmetry(cell, symprec=1e-5)
        print("CRYSTAL-interface: Magnetic structure, number of operations without spin: %d" %
              len(symmetry.get_symmetry_operations()['rotations']))
        print("CRYSTAL-interface: Spacegroup without spin: %s" % symmetry.get_international_table())

        cell.set_magnetic_moments(magmoms)
        symmetry = Symmetry(cell, symprec=1e-5)
        print("CRYSTAL-interface: Magnetic structure, number of operations with spin: %d" %
              len(symmetry.get_symmetry_operations()['rotations']))
        print("")

    return cell, tags['conv_numbers']

def write_crystal(filename, cell, conv_numbers, template_file="TEMPLATE", write_symmetry=False):
    # Write geometry in EXTERNAL file (fort.34)
    f_ext = open(filename + '.ext', 'w')
    f_ext.write(get_crystal_structure(cell, conv_numbers, write_symmetry))
    f_ext.close()

    # Create input file (filename.d12)
    lines = "Created by Phonopy CRYSTAL interface\n"
    lines += "EXTERNAL\n"
    lines += "ENDGEOM\n"
    # If template_file exists, insert it at this point
    try:
        f_template = open(template_file)
        lines += f_template.read()
        f_template.close()
    except IOError:
        lines += "***** Insert basis sets and parameters here *****\n"
    lines += "TOLDEE\n"
    lines += "10\n"
    # For magnetic structures, create ATOMSPIN entry
    # Only spins != 0 are written
    magmoms = cell.get_magnetic_moments()
    if magmoms is not None:
        atomspins = ""
        N_spins = 0
        for i in range(0, len(magmoms)):
            if magmoms[i] != 0:
                N_spins += 1
                atomspins += ("%d %d " % (i + 1, magmoms[i]))
        lines += "ATOMSPIN\n"
        lines += ("%d\n" % N_spins)
        lines += atomspins + "\n"
    lines += "GRADCAL\n"
    lines += "END\n"

    # Write the input file
    f_inputfile = open(filename + '.d12', 'w')
    f_inputfile.writelines(lines)
    f_inputfile.close()

def write_supercells_with_displacements(supercell,
                                        cells_with_displacements,
                                        conv_numbers,
                                        num_unitcells_in_supercell,
                                        pre_filename="supercell",
                                        width=3,
                                        template_file="TEMPLATE"):
    convnum_super = []
    for i in conv_numbers:
        for j in range(num_unitcells_in_supercell):
            convnum_super.append(i)

    # Currently, symmetry is not used by default
    # It can be turned on by creating a file called CRY_SYM
    try:
        f = open('CRY_SYM')
        use_symmetry = True
        f.close()
    except IOError:
        use_symmetry = False

    if use_symmetry:
        print("CRYSTAL-interface: WARNING: Symmetry enabled in EXTERNAL files.\n" +
              "  Check the supercells very carefully, some spacegroups do not work (e.g. R-3m)\n" +
              "  Non-displaced supercell is always written without symmetry")

    write_crystal("supercell", supercell, convnum_super, template_file, write_symmetry=False)
    for i, cell in enumerate(cells_with_displacements):
        if cell is not None:
            filename = "{pre_filename}-{0:0{width}}".format(
                       i + 1,
                       pre_filename=pre_filename,
                       width=width)
            write_crystal(filename, cell, convnum_super, template_file, write_symmetry=use_symmetry)

def get_crystal_structure(cell, conv_numbers, write_symmetry=False):
    lattice = cell.get_cell()
    positions = cell.get_positions()
    numbers = cell.get_atomic_numbers()

    # Create and EXTERNAL file (fort.34)
    # Dimensionality, centring, crystal type
    lines = "3 1 1\n"
    # Cartesian components of the lattice vectors
    for lattvec in lattice:
      lines += ("%12.8f" * 3 + "\n") % tuple(lattvec)

    # Symmetry operators
    if write_symmetry:
        symmetry = Symmetry(cell, symprec=1e-5)
        rotations = symmetry.get_symmetry_operations()['rotations']
        translations = symmetry.get_symmetry_operations()['translations']
        N_symmops = 0
        symmlines = ""
        for i in range(0, len(rotations)):
            N_symmops += 1
            for j in range(0, 3):
                symmlines += ("  %5.2f" * 3 + "\n") % tuple(rotations[i][j])
            symmlines += ("  %5.2f" * 3 + "\n") % tuple(translations[i])

        lines += ("%d\n" % N_symmops)
        lines += symmlines

    else:
        lines += "1\n"
        lines += "  1.00  0.00  0.00\n"
        lines += "  0.00  1.00  0.00\n"
        lines += "  0.00  0.00  1.00\n"
        lines += "  0.00  0.00  0.00\n"

    # Number of atoms in the unit cell (asymmetric unit)
    lines += ("%d\n") % len(positions)
    # Conventional atomic number and cartesian coordinates of the atoms
    for i, pos in zip(conv_numbers, positions):
        lines += ("  %d " + "%16.12f"*3 + "\n") % (i,  pos[0], pos[1], pos[2])

    return lines

class CrystalIn:
    def __init__(self, lines):
        # conv_numbers = CRYSTAL conventional atomic number mapping: 'Ge' -> 32 or 'Ge' -> 232
        self._tags = {'lattice_vectors':  None,
                      'atomic_species':   None,
                      'coordinates':      None,
                      'magnetic_moments': None,
                      'conv_numbers':     None}

        self._values = None
        self._collect(lines)

    def get_tags(self):
        return self._tags

    def _collect(self, lines):
        # Reads a CRYSTAL output file (lattice vectors, conventional atomic numbers,
        # fractional atomic positions).
        # - For optimization outputs, the final geometry in the file is read.
        # - Dielectric tensor and effective Born charges can be read with script phonopy-crystal-born
        # - If ATOMSPIN keyword is present, magnetic moments are read from it
        magmoms = []
        atomspins = []
        numspins = 0
        l = 0
        while l < len(lines):
            line = lines[l]
            if 'PRIMITIVE CELL - CENTRING CODE' in line:
                aspecies = []
                coords = []
                convnum = []
                l += 4
                # ATOMS IN THE ASYMMETRIC UNIT    2 - ATOMS IN THE UNIT CELL:    6
                N_asym_atoms = int(lines[l].split()[5])
                N_atoms = int(lines[l].split()[12])
                l += 3
                # 1 T  22 TI    4.721218104494E-21  3.307446203077E-21  1.413771901417E-21
                for atom in range(0, N_atoms):
                    atomdata = lines[l].split()
                    aspecies.append(atomdata[3].capitalize())
                    coords.append([float(x) for x in atomdata[4:7]])
                    convnum.append(int(atomdata[2]))
                    l += 1
            elif 'DIRECT LATTICE VECTORS CARTESIAN COMPONENTS' in line:
                lattvecs = []
                l += 2
                #          X                    Y                    Z
                for lattvec in range(1, 4):
                    lattvecs.append([float(x) for x in lines[l].split()])
                    l += 1
            elif 'ATOMSPIN' in line:
                # Read ATOMSPIN, and save the magnetic moments for later parsing
                # (not all necessary information is available at this point)
                # All spins must be entered on one line!
                # ATOMSPIN
                # 8
                # 1 1 2 1 3 -1 4 -1 5 1 6 1 7 -1 8 -1
                l += 1
                numspins = int(lines[l])
                l += 1
                atomspins = [int(x) for x in lines[l].split()]
                l += 1

            l += 1 # while l < len(lines)

        if len(lattvecs) == 3 and len(aspecies) > 0 and len(aspecies) == len(coords) and len(aspecies) == len(convnum):
            self._tags['lattice_vectors'] = lattvecs
            self._tags['atomic_species'] = aspecies
            self._tags['coordinates'] = coords
            self._tags['conv_numbers'] = convnum
        else:
            print("CRYSTAL-interface: Error parsing CRYSTAL output file")

        # Set magnetic moments
        if numspins > 0:
            # Initialize all moments to zero
            magmoms = [0] * N_atoms
            if numspins * 2 == len(atomspins):
                for i in range(0, numspins):
                    atomnum = atomspins[i*2] - 1
                    magmom = atomspins[i*2 + 1]
                    magmoms[atomnum] = magmom

                self._tags['magnetic_moments'] = magmoms
                print("CRYSTAL-interface: Following magnetic moments have been read from ATOMSPIN entry:")
                print(magmoms)
            else:
                print("CRYSTAL-interface: Invalid ATOMSPIN entry, magnetic moments have not been set")
        else:
            print("")


if __name__ == '__main__':
    import sys
    from phonopy.structure.symmetry import Symmetry
    cell, conv_numbers = read_crystal(sys.argv[1])
    symmetry = Symmetry(cell)
    print("# %s" % symmetry.get_international_table())
    print(get_crystal_structure(cell, conv_numbers))



# Copyright (C) 2018 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import numpy as np
from phonopy.interface.calculator import read_crystal_structure
from phonopy.structure.cells import get_primitive_matrix_by_centring
from phonopy.file_IO import (
    parse_BORN, read_force_constants_hdf5,
    read_physical_unit_in_force_constants_hdf5)
from phonopy.structure.atoms import PhonopyAtoms
from phonopy.interface.calculator import get_force_constant_conversion_factor


def get_cell_settings(phonopy_yaml=None,
                      supercell_matrix=None,
                      primitive_matrix=None,
                      unitcell=None,
                      supercell=None,
                      unitcell_filename=None,
                      supercell_filename=None,
                      calculator=None,
                      symprec=1e-5):
    if unitcell_filename is not None:
        cell, filename = read_crystal_structure(
            filename=unitcell_filename, interface_mode=calculator)
        smat = get_supercell_matrix(supercell_matrix)
        pmat = primitive_matrix
    elif supercell_filename is not None:
        cell, filename = read_crystal_structure(
            filename=supercell_filename, interface_mode=calculator)
        smat = np.eye(3, dtype='intc', order='C')
        if primitive_matrix is None or primitive_matrix == "auto":
            pmat = 'auto'
    elif unitcell is not None:
        cell = PhonopyAtoms(atoms=unitcell)
        smat = get_supercell_matrix(supercell_matrix)
        pmat = primitive_matrix
    elif supercell is not None:
        cell = PhonopyAtoms(atoms=supercell)
        smat = np.eye(3, dtype='intc', order='C')
        if primitive_matrix is None or primitive_matrix == "auto":
            pmat = 'auto'
    else:
        raise RuntimeError("Cell has to be specified.")

    if cell is None:
        msg = "'%s' could not be found." % filename
        raise FileNotFoundError(msg)

    pmat = _get_primitive_matrix(pmat, cell, symprec)

    return cell, smat, pmat


def get_nac_params(primitive, nac_params, born_filename, is_nac, nac_factor):
    _nac_params = None
    if nac_params is not None:
        _nac_params = nac_params
    elif born_filename is not None:
        _nac_params = parse_BORN(primitive, filename=born_filename)
    elif is_nac is True:
        if os.path.isfile("BORN"):
            _nac_params = parse_BORN(primitive, filename="BORN")

    if _nac_params is not None:
        if _nac_params['factor'] is None:
            _nac_params['factor'] = nac_factor

    return _nac_params


def read_force_constants_from_hdf5(filename='force_constants.hdf5',
                                   p2s_map=None,
                                   calculator=None):
    """Convert force constants physical unit

    Each calculator interface has own default force constants physical unit.
    This method reads the physical unit of force constants if it exists and
    if the read physical unit is different from the default one, the force
    constants values are converted to have the default physical units.

    Note
    ----
    This method is also used from phonopy script.

    """

    fc = read_force_constants_hdf5(filename=filename, p2s_map=p2s_map)
    fc_unit = read_physical_unit_in_force_constants_hdf5(
        filename=filename)
    if fc_unit is None:
        return fc
    else:
        factor = get_force_constant_conversion_factor(fc_unit, calculator)
        return fc * factor


def get_supercell_matrix(smat):
    if smat is None:
        _smat = np.eye(3, dtype='intc', order='C')
    elif len(np.ravel(smat)) == 3:
        _smat = np.diag(smat)
    elif len(np.ravel(smat)) == 9:
        _smat = np.reshape(smat, (3, 3))
    else:
        msg = "supercell_matrix shape has to be (3,) or (3, 3)"
        raise RuntimeError(msg)
    return _smat


def _get_primitive_matrix(pmat, unitcell, symprec):
    if type(pmat) is str and pmat in ('F', 'I', 'A', 'C', 'R', 'auto'):
        if pmat == 'auto':
            _pmat = pmat
        else:
            _pmat = get_primitive_matrix_by_centring(pmat)
    elif pmat is None:
        _pmat = None
    elif len(np.ravel(pmat)) == 9:
        matrix = np.reshape(pmat, (3, 3))
        if matrix.dtype.kind in ('i', 'u', 'f'):
            det = np.linalg.det(matrix)
            if symprec < det and det < 1 + symprec:
                _pmat = matrix
            else:
                msg = ("Determinant of primitive_matrix has to be larger "
                       "than 0")
                raise RuntimeError(msg)
    else:
        msg = ("primitive_matrix has to be a 3x3 matrix, None, 'auto', "
               "'F', 'I', 'A', 'C', or 'R'")
        raise RuntimeError(msg)

    return _pmat



# Copyright (C) 2018 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import numpy as np
from phonopy.api_phonopy import Phonopy
from phonopy.file_IO import parse_FORCE_SETS, parse_FORCE_CONSTANTS
from phonopy.interface.phonopy_yaml import PhonopyYaml
from phonopy.interface.calculator import get_default_physical_units
import phonopy.cui.load_helper as load_helper


def load(phonopy_yaml=None,  # phonopy.yaml-like must be the first argument.
         supercell_matrix=None,
         primitive_matrix=None,
         is_nac=True,
         calculator=None,
         unitcell=None,
         supercell=None,
         nac_params=None,
         unitcell_filename=None,
         supercell_filename=None,
         born_filename=None,
         force_sets_filename=None,
         force_constants_filename=None,
         fc_calculator=None,
         factor=None,
         frequency_scale_factor=None,
         symprec=1e-5,
         is_symmetry=True,
         log_level=0):
    """Create Phonopy instance from parameters and/or input files.

    When unitcell and unitcell_filename are not given, file name that is
    default for the chosen calculator is looked for in the current directory
    as the default behaviour.

    When force_sets_filename and force_constants_filename are not given,
    'FORCE_SETS' is looked for in the current directory as the default
    behaviour.

    Parameters
    ----------
    phonopy_yaml : str, optional
        Filename of "phonopy.yaml"-like file. If this is given, the data
        in the file are parsed. Default is None.
    supercell_matrix : array_like, optional
        Supercell matrix multiplied to input cell basis vectors.
        shape=(3, ) or (3, 3), where the former is considered a diagonal
        matrix. Default is the unit matrix.
        dtype=int
    primitive_matrix : array_like or str, optional
        Primitive matrix multiplied to input cell basis vectors. Default is
        the identity matrix.
        shape=(3, 3)
        dtype=float
        When 'F', 'I', 'A', 'C', or 'R' is given instead of a 3x3 matrix,
        the primitive matrix defined at
        https://atztogo.github.io/spglib/definition.html
        is used.
    is_nac : bool, optional
        If True, look for 'BORN' file. If False, NAS is turned off.
        The priority for NAC is nac_params > born_filename > is_nac ('BORN').
        Default is True.
    calculator : str, optional.
        Calculator used for computing forces. This is used to switch the set
        of physical units. Default is None, which is equivalent to "vasp".
    unitcell : PhonopyAtoms, optional
        Input unit cell. Default is None. The priority for cell is
        unitcell_filename > supercell_filename > unitcell > supercell.
    supercell : PhonopyAtoms, optional
        Input supercell cell. Default value of primitive_matrix is set to
        'auto' (can be overwitten). supercell_matrix is ignored. Default is
        None. The priority for cell is
        unitcell_filename > supercell_filename > unitcell > supercell.
    nac_params : dict, optional
        Parameters required for non-analytical term correction. Default is
        None. The priority for NAC is nac_params > born_filename > is_nac.
        {'born': Born effective charges
                 (array_like, shape=(primitive cell atoms, 3, 3), dtype=float),
         'dielectric': Dielectric constant matrix
                       (array_like, shape=(3, 3), dtype=float),
         'factor': unit conversion facotr (float)}
    unitcell_filename : str, optional
        Input unit cell filename. Default is None. The priority for cell is
        unitcell_filename > supercell_filename > unitcell > supercell.
    supercell_filename : str, optional
        Input supercell filename. When this is specified, supercell_matrix is
        ignored. Default is None. The priority for cell is
        1. unitcell_filename (with supercell_matrix)
        2. supercell_filename
        3. unitcell (with supercell_matrix)
        4. supercell.
    born_filename : str, optional
        Filename corresponding to 'BORN', a file contains non-analytical term
        correction parameters.
        The priority for NAC is nac_params > born_filename > is_nac ('BORN').
    force_sets_filename : str, optional
        Filename of a file corresponding to 'FORCE_SETS', a file contains sets
        of forces and displacements. Default is None.
        The priority for force constants is
        force_constants_filename > force_sets_filename > 'FORCE_SETS'.
    force_constants_filename : str, optional
        Filename of a file corresponding to 'FORCE_CONSTANTS' or
        'force_constants.hdf5', a file contains force constants.
        Default is None.
        The priority for force constants is
        force_constants_filename > force_sets_filename > 'FORCE_SETS'.
    fc_calculator : str, optional
        Force constants calculator. Currently only 'alm'. Default is None.
    factor : float, optional
        Phonon frequency unit conversion factor. Unless specified, default
        unit conversion factor for each calculator is used.
    frequency_scale_factor : float, optional
        Factor multiplied to calculated phonon frequency. Default is None,
        i.e., effectively 1.
    symprec : float, optional
        Tolerance used to find crystal symmetry. Default is 1e-5.
    is_symmetry : bool, optional
        Setting False, crystal symmetry except for lattice translation is not
        considered. Default is True.
    log_level : int, optional
        Verbosity control. Default is 0.

    """

    if phonopy_yaml is None:
        cell, smat, pmat = load_helper.get_cell_settings(
            supercell_matrix=supercell_matrix,
            primitive_matrix=primitive_matrix,
            unitcell=unitcell,
            supercell=supercell,
            unitcell_filename=unitcell_filename,
            supercell_filename=supercell_filename,
            calculator=calculator,
            symprec=symprec)
        _nac_params = nac_params
        _dataset = None
        _fc = None
    else:
        phpy_yaml = PhonopyYaml()
        phpy_yaml.read(phonopy_yaml)
        cell = phpy_yaml.unitcell
        smat = phpy_yaml.supercell_matrix
        if smat is None:
            smat = np.eye(3, dtype='intc', order='C')
        if primitive_matrix == 'auto':
            pmat = 'auto'
        else:
            pmat = phpy_yaml.primitive_matrix
        if is_nac:
            _nac_params = phpy_yaml.nac_params
        else:
            _nac_params = None
        _dataset = phpy_yaml.dataset
        _fc = phpy_yaml.force_constants

    # units keywords: factor, nac_factor, distance_to_A
    units = get_default_physical_units(calculator)
    if factor is None:
        _factor = units['factor']
    else:
        _factor = factor
    phonon = Phonopy(cell,
                     smat,
                     primitive_matrix=pmat,
                     factor=_factor,
                     frequency_scale_factor=frequency_scale_factor,
                     symprec=symprec,
                     is_symmetry=is_symmetry,
                     calculator=calculator,
                     log_level=log_level)
    _nac_params = load_helper.get_nac_params(phonon.primitive,
                                             _nac_params,
                                             born_filename,
                                             is_nac,
                                             units['nac_factor'])
    if _nac_params is not None:
        phonon.nac_params = _nac_params

    if _fc is None:
        _set_force_constants(
            phonon,
            dataset=_dataset,
            force_constants_filename=force_constants_filename,
            force_sets_filename=force_sets_filename,
            calculator=calculator,
            fc_calculator=fc_calculator)
    else:
        phonon.force_constants = _fc
    return phonon


def _set_force_constants(
        phonon,
        dataset=None,
        force_constants_filename=None,
        force_sets_filename=None,
        calculator=None,
        fc_calculator=None):
    natom = phonon.supercell.get_number_of_atoms()

    _dataset = None
    if dataset is not None:
        _dataset = dataset
    elif force_constants_filename is not None:
        dot_split = force_constants_filename.split('.')
        p2s_map = phonon.primitive.p2s_map
        if len(dot_split) > 1 and dot_split[-1] == 'hdf5':
            fc = load_helper.read_force_constants_from_hdf5(
                filename=force_constants_filename,
                p2s_map=p2s_map,
                calculator=calculator)
        else:
            fc = parse_FORCE_CONSTANTS(filename=force_constants_filename,
                                       p2s_map=p2s_map)
        phonon.set_force_constants(fc)
    elif force_sets_filename is not None:
        _dataset = parse_FORCE_SETS(natom=natom,
                                    filename=force_sets_filename)
    elif os.path.isfile("FORCE_SETS"):
        _dataset = parse_FORCE_SETS(natom=natom)

    if _dataset:
        phonon.dataset = _dataset
        try:
            phonon.produce_force_constants(
                calculate_full_force_constants=False,
                fc_calculator=fc_calculator)
        except RuntimeError:
            pass



import unittest
import numpy as np
from phonopy.structure.brillouin_zone import BrillouinZone
from phonopy.structure.spglib import (get_stabilized_reciprocal_mesh,
                                      relocate_BZ_grid_address)


class TestBrillouinZone(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_FCC(self):
        direct_lat = [[0, 2.73, 2.73], [2.73, 0, 2.73], [2.73, 2.73, 0]]
        is_shift = [0, 0, 0]
        self._testBrillouinZone(direct_lat, [4, 4, 4], is_shift)
        self._testBrillouinZone(direct_lat, [5, 5, 5], is_shift)

    def test_Hexagonal(self):
        direct_lat = [[3.0751691007292523, 0, 0],
                      [-1.5375845503646262, 2.6631745621644800, 0],
                      [0, 0, 3.5270080068586522]]
        is_shift = [0, 0, 0]
        self._testBrillouinZone(direct_lat, [4, 4, 4], is_shift)
        self._testBrillouinZone(direct_lat, [5, 5, 5], is_shift)

    def _testBrillouinZone(self, direct_lat, mesh, is_shift):
        _, grid_address = get_stabilized_reciprocal_mesh(
            mesh,
            rotations=[np.eye(3, dtype='intc'), ],
            is_shift=is_shift)
        rec_lat = np.linalg.inv(direct_lat)
        bz_grid_address, bz_map = relocate_BZ_grid_address(
            grid_address,
            mesh,
            rec_lat,
            is_shift=is_shift)

        qpoints = (grid_address + np.array(is_shift) / 2.0) / mesh
        bz = BrillouinZone(rec_lat)
        bz.run(qpoints)
        sv_all = bz.shortest_qpoints  # including BZ boundary duplicates
        sv = [v[0] for v in sv_all]
        bz_qpoints = (bz_grid_address + np.array(is_shift) / 2.0) / mesh
        d2_this = (np.dot(sv, rec_lat.T) ** 2).sum(axis=1)
        d2_spglib = (np.dot(bz_qpoints[:np.prod(mesh)],
                            rec_lat.T) ** 2).sum(axis=1)
        diff = d2_this - d2_spglib
        diff -= np.rint(diff)

        # Following both of two tests are necessary.
        # Check equivalence of vectors by lattice translation
        np.testing.assert_allclose(diff, 0, atol=1e-8)
        # Check being in same (hopefull first) Brillouin zone by their lengths
        np.testing.assert_allclose(d2_this, d2_spglib, atol=1e-8)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestBrillouinZone)
    unittest.TextTestRunner(verbosity=2).run(suite)



import unittest

from phonopy.structure.atoms import PhonopyAtoms


class TestCell(unittest.TestCase):

    def setUp(self):
        symbols = ['Si'] * 2 + ['O'] * 4
        lattice = [[4.65, 0, 0],
                   [0, 4.75, 0],
                   [0, 0, 3.25]]
        points = [[0.0, 0.0, 0.0],
                  [0.5, 0.5, 0.5],
                  [0.3, 0.3, 0.0],
                  [0.7, 0.7, 0.0],
                  [0.2, 0.8, 0.5],
                  [0.8, 0.2, 0.5]]

        self._cells = []
        self._cells.append(PhonopyAtoms(cell=lattice,
                                        scaled_positions=points,
                                        symbols=symbols))

        # The element for which mass is not defined.
        symbols = ['Ac'] * 2 + ['O'] * 4
        self._cells.append(PhonopyAtoms(cell=lattice,
                                        scaled_positions=points,
                                        symbols=symbols))

    def tearDown(self):
        pass

    def test_atoms(self):
        for cell in self._cells:
            print(cell.get_cell())
            for s, p in zip(cell.get_chemical_symbols(),
                            cell.get_scaled_positions()):
                print("%s %s" % (s, p))

    def test_phonopy_atoms(self):
        for cell in self._cells:
            print(PhonopyAtoms(atoms=cell))


if __name__ == '__main__':
    unittest.main()



import phonopy
from phonopy.phonon.band_structure import get_band_qpoints

phonon = phonopy.load([[2, 0, 0], [0, 2, 0], [0, 0, 2]],
                      primitive_matrix=[[0, 0.5, 0.5],
                                        [0.5, 0, 0.5],
                                        [0.5, 0.5, 0]],
                      unitcell_filename="POSCAR",
                      force_sets_filename="FORCE_SETS",
                      born_filename="BORN")
points = get_band_qpoints(
    [[[0.5, 0, 0.5], [0, 0, 0], [0.5, 0.5, 0.5], [0.5, 0.25, 0.75]]], 51)
phonon.set_band_structure(points)
phonon.plot_band_structure(labels=['X', '$\Gamma$', 'L', 'W']).show()



import unittest
import os
import numpy as np
from phonopy import Phonopy
from phonopy.interface.vasp import read_vasp
from phonopy.file_IO import parse_FORCE_SETS, parse_BORN
from phonopy.units import VaspToTHz

data_dir = os.path.dirname(os.path.abspath(__file__))


class TestQpoints(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def _get_phonon(self):
        cell = read_vasp(os.path.join(data_dir, "..", "POSCAR_NaCl"))
        phonon = Phonopy(cell,
                         np.diag([2, 2, 2]),
                         primitive_matrix=[[0, 0.5, 0.5],
                                           [0.5, 0, 0.5],
                                           [0.5, 0.5, 0]])
        filename = os.path.join(data_dir, "..", "FORCE_SETS_NaCl")
        force_sets = parse_FORCE_SETS(filename=filename)
        phonon.dataset = force_sets
        phonon.produce_force_constants()
        filename_born = os.path.join(data_dir, "..", "BORN_NaCl")
        nac_params = parse_BORN(phonon.get_primitive(), filename=filename_born)
        phonon.nac_params = nac_params
        return phonon

    def testQpoints(self):
        phonon = self._get_phonon()
        phonon.run_qpoints([0, 0, 0], with_eigenvectors=True)
        qpoints_phonon = phonon.qpoints
        phonon.get_qpoints_dict()
        np.testing.assert_allclose(qpoints_phonon.eigenvectors,
                                   qpoints_phonon.get_eigenvectors())
        np.testing.assert_allclose(qpoints_phonon.frequencies,
                                   qpoints_phonon.get_frequencies())
        phonon.run_qpoints([0, 0, 0], with_dynamical_matrices=True)
        dm = phonon.qpoints.dynamical_matrices
        eigs = np.linalg.eigvalsh(dm) * VaspToTHz ** 2
        freqs = phonon.qpoints.frequencies
        np.testing.assert_allclose(freqs ** 2 * np.sign(freqs), eigs)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestQpoints)
    unittest.TextTestRunner(verbosity=2).run(suite)



import unittest
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
import numpy as np
from phonopy import Phonopy
from phonopy.interface.vasp import read_vasp
from phonopy.file_IO import parse_FORCE_SETS
from phonopy.phonon.tetrahedron_mesh import TetrahedronMesh
import os

data_dir = os.path.dirname(os.path.abspath(__file__))

dos_str = """-0.672024 0.000000 0.029844 0.005522 0.731712 0.029450 1.433580 0.116998
2.135448 0.200612 2.837315 0.329781 3.539183 0.905737 4.241051 1.808068
4.942919 1.921464 5.644787 1.415087 6.346654 1.281276 7.048522 1.127877
7.750390 0.725551 8.452258 1.611979 9.154126 1.157525 9.855993 0.929781
10.557861 0.705942 11.259729 0.630369 11.961597 0.557250 12.663465 0.487940
13.365332 0.143148 14.067200 0.231518 14.769068 0.205816 15.470936 0.374575
16.172804 0.781758 16.874671 0.352501 17.576539 0.473537 18.278407 0.310505
18.980275 0.184665 19.682143 0.018812 20.384010 0.023455 21.085878 0.028958
21.787746 0.035963 22.489614 0.047177 23.191481 0.068537 23.893349 0.092864
24.595217 0.204422 25.297085 0.252177 25.998953 0.676881 26.700820 0.000000"""


class TestTetrahedronMesh(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_Amm2(self):
        data = np.loadtxt(StringIO(dos_str))
        phonon = self._get_phonon("Amm2",
                                  [3, 2, 2],
                                  [[1, 0, 0],
                                   [0, 0.5, -0.5],
                                   [0, 0.5, 0.5]])
        mesh = [11, 11, 11]
        primitive = phonon.get_primitive()
        phonon.run_mesh([11, 11, 11])
        qpoints = phonon.mesh.qpoints
        weights = phonon.mesh.weights
        frequencies = phonon.mesh.frequencies
        grid_address = phonon.mesh.grid_address
        ir_grid_points = phonon.mesh.ir_grid_points
        grid_mapping_table = phonon.mesh.grid_mapping_table
        thm = TetrahedronMesh(primitive,
                              frequencies,
                              mesh,
                              grid_address,
                              grid_mapping_table,
                              ir_grid_points)
        thm.set(value='I', division_number=40)
        freq_points = thm.get_frequency_points()
        dos = np.zeros_like(freq_points)

        for i, iw in enumerate(thm):
            dos += np.sum(iw * weights[i], axis=1)

        dos_comp = np.transpose([freq_points, dos]).reshape(10, 8)
        self.assertTrue(np.abs(dos_comp - data).all() < 1e-5)

    def _show(self, freq_points, dos):
        data = []
        for f, d in zip(freq_points, dos):
            data.append([f, d])
        data = np.reshape(data, (10, 8))
        for row in data:
            print(("%f " * 8) % tuple(row))

    def _get_phonon(self, spgtype, dim, pmat):
        cell = read_vasp(os.path.join(data_dir, "POSCAR_%s" % spgtype))
        phonon = Phonopy(cell,
                         np.diag(dim),
                         primitive_matrix=pmat)
        force_sets = parse_FORCE_SETS(
            filename=os.path.join(data_dir, "FORCE_SETS_%s" % spgtype))
        phonon.set_displacement_dataset(force_sets)
        phonon.produce_force_constants()
        return phonon


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestTetrahedronMesh)
    unittest.TextTestRunner(verbosity=2).run(suite)



import unittest
import os
import numpy as np
import phonopy

data_dir = os.path.dirname(os.path.abspath(__file__))

tp_str = """0.000000 100.000000 200.000000 300.000000 400.000000
500.000000 600.000000 700.000000 800.000000 900.000000
4.856373 3.916036 -0.276031 -6.809284 -14.961974
-24.342086 -34.708562 -45.898943 -57.796507 -70.313405
0.000000 26.328820 55.258118 74.269718 88.156943
99.054231 108.009551 115.606163 122.200204 128.024541
0.000000 36.207859 45.673598 47.838871 48.634251
49.009334 49.214960 49.339592 49.420745 49.476502"""

tdos_str = """-0.750122 0.000000
0.249878 0.000041
1.249878 0.010398
2.249878 1.363019
3.249878 1.837252
4.249878 0.991171
5.249878 0.342658
6.249878 0.106961
7.249878 0.032966"""

tdos_thm_str = """-0.750122 0.000000
0.249878 0.004508
1.249878 0.090716
2.249878 0.549237
3.249878 1.093288
4.249878 1.349766
5.249878 0.660090
6.249878 0.461219
7.249878 0.000000"""

pdos_str = """-0.750122 0.000000 0.000000
0.249878 0.000016 0.000025
1.249878 0.004469 0.005929
2.249878 0.620921 0.742098
3.249878 0.597587 1.239666
4.249878 0.501077 0.490094
5.249878 0.142162 0.200496
6.249878 0.070026 0.036935
7.249878 0.020225 0.012741"""

pdos_thm_str = """-0.750122 0.000000 0.000000
0.249878 0.001820 0.002688
1.249878 0.037419 0.053297
2.249878 0.228129 0.321109
3.249878 0.403637 0.689651
4.249878 0.756532 0.593234
5.249878 0.454573 0.205518
6.249878 0.322527 0.138692
7.249878 0.000000 0.000000"""


class TestDOS(unittest.TestCase):
    def setUp(self):
        self._phonon = self._get_phonon()

    def tearDown(self):
        pass

    def testTotalDOS(self):
        phonon = self._phonon
        phonon.run_mesh([5, 5, 5])
        phonon.run_total_dos(freq_pitch=1, use_tetrahedron_method=False)
        dos = phonon.total_dos.dos
        freqs = phonon.total_dos.frequency_points
        data_ref = np.reshape([float(x) for x in tdos_str.split()], (-1, 2))
        np.testing.assert_allclose(data_ref, np.c_[freqs, dos], atol=1e-5)
        # for f, d in zip(freqs, dos):
        #     print("%f %f" % (f, d))

    def testTotalDOSTetrahedron(self):
        phonon = self._phonon
        phonon.run_mesh([5, 5, 5])
        phonon.run_total_dos(freq_pitch=1, use_tetrahedron_method=True)
        dos = phonon.total_dos.dos
        freqs = phonon.total_dos.frequency_points
        data_ref = np.reshape([float(x) for x in tdos_thm_str.split()],
                              (-1, 2))
        np.testing.assert_allclose(data_ref, np.c_[freqs, dos], atol=1e-5)
        # for f, d in zip(freqs, dos):
        #     print("%f %f" % (f, d))

    def testPartialDOS(self):
        phonon = self._phonon
        phonon.run_mesh([5, 5, 5],
                        is_mesh_symmetry=False,
                        with_eigenvectors=True)
        phonon.run_projected_dos(freq_pitch=1, use_tetrahedron_method=False)
        pdos = phonon.projected_dos.projected_dos
        freqs = phonon.projected_dos.frequency_points
        data_ref = np.reshape([float(x) for x in pdos_str.split()],
                              (-1, 3)).T
        np.testing.assert_allclose(data_ref, np.vstack([freqs, pdos]),
                                   atol=1e-5)
        # for f, d in zip(freqs, pdos.T):
        #     print(("%f" + " %f" * len(d)) % ((f, ) + tuple(d)))

    def testPartialDOSTetrahedron(self):
        phonon = self._phonon
        phonon.run_mesh([5, 5, 5],
                        is_mesh_symmetry=False,
                        with_eigenvectors=True)
        phonon.run_projected_dos(freq_pitch=1, use_tetrahedron_method=True)
        pdos = phonon.projected_dos.projected_dos
        freqs = phonon.projected_dos.frequency_points
        data_ref = np.reshape([float(x) for x in pdos_thm_str.split()],
                              (-1, 3)).T
        np.testing.assert_allclose(data_ref, np.vstack([freqs, pdos]),
                                   atol=1e-5)
        # for f, d in zip(freqs, pdos.T):
        #     print(("%f" + " %f" * len(d)) % ((f, ) + tuple(d)))

    def _get_phonon(self):
        phonon = phonopy.load(
            supercell_matrix=[[2, 0, 0], [0, 2, 0], [0, 0, 2]],
            primitive_matrix=[[0, 0.5, 0.5],
                              [0.5, 0, 0.5],
                              [0.5, 0.5, 0]],
            unitcell_filename=os.path.join(data_dir, "..", "POSCAR_NaCl"),
            force_sets_filename=os.path.join(data_dir, "..",
                                             "FORCE_SETS_NaCl"),
            born_filename=os.path.join(data_dir, "..", "BORN_NaCl"))
        return phonon


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestDOS)
    unittest.TextTestRunner(verbosity=2).run(suite)



import unittest

import os
import numpy as np
from phonopy.structure.atoms import PhonopyAtoms as Atoms
from phonopy.structure.cells import get_supercell, get_primitive
from phonopy.interface.phonopy_yaml import read_cell_yaml

data_dir = os.path.dirname(os.path.abspath(__file__))


class TestSupercell(unittest.TestCase):

    def setUp(self):
        self._cells = []
        symbols = ['Si'] * 2 + ['O'] * 4
        lattice = [[4.65, 0, 0],
                   [0, 4.75, 0],
                   [0, 0, 3.25]]
        points = [[0.0, 0.0, 0.0],
                  [0.5, 0.5, 0.5],
                  [0.3, 0.3, 0.0],
                  [0.7, 0.7, 0.0],
                  [0.2, 0.8, 0.5],
                  [0.8, 0.2, 0.5]]

        self._cells.append(Atoms(cell=lattice,
                                 scaled_positions=points,
                                 symbols=symbols))

        symbols = ['Si'] * 2
        lattice = [[0, 2.73, 2.73],
                   [2.73, 0, 2.73],
                   [2.73, 2.73, 0]]
        points = [[0.75, 0.75, 0.75],
                  [0.5, 0.5, 0.5]]

        self._cells.append(Atoms(cell=lattice,
                                 scaled_positions=points,
                                 symbols=symbols))

        self._smats = []
        self._smats.append(np.diag([1, 2, 3]))
        self._smats.append([[-1, 1, 1],
                            [1, -1, 1],
                            [1, 1, -1]])

        self._fnames = ("SiO2-123.yaml", "Si-conv.yaml")

    def tearDown(self):
        pass

    def test_get_supercell(self):
        for i, (cell, smat, fname) in enumerate(zip(self._cells,
                                                    self._smats,
                                                    self._fnames)):
            scell = get_supercell(cell, smat)
            scell_yaml = read_cell_yaml(os.path.join(data_dir, fname))
            np.testing.assert_allclose(scell.get_cell(), scell_yaml.get_cell(),
                                       atol=1e-5)
            pos = scell.get_scaled_positions()
            pos -= np.rint(pos)
            pos_yaml = scell_yaml.get_scaled_positions()
            pos_yaml -= np.rint(pos_yaml)
            np.testing.assert_allclose(pos, pos_yaml, atol=1e-5)
            np.testing.assert_array_equal(scell.get_atomic_numbers(),
                                          scell_yaml.get_atomic_numbers())
            np.testing.assert_allclose(scell.get_masses(),
                                       scell_yaml.get_masses(),
                                       atol=1e-5)


class TestPrimitive(unittest.TestCase):

    def setUp(self):
        cell = read_cell_yaml(os.path.join(data_dir, "..", "NaCl.yaml"))
        self._pcell = get_primitive(cell, [[0, 0.5, 0.5],
                                           [0.5, 0, 0.5],
                                           [0.5, 0.5, 0]])

    def tearDown(self):
        pass

    def test_properties(self):
        np.testing.assert_array_equal(
            self._pcell.p2s_map,
            self._pcell.get_primitive_to_supercell_map())
        np.testing.assert_array_equal(
            self._pcell.s2p_map,
            self._pcell.get_supercell_to_primitive_map())
        np.testing.assert_array_equal(
            self._pcell.atomic_permutations,
            self._pcell.get_atomic_permutations())
        self.assertTrue(id(self._pcell.p2p_map)
                        == id(self._pcell.get_primitive_to_primitive_map()))



if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestSupercell)
    suite = unittest.TestLoader().loadTestsFromTestCase(TestPrimitive)
    unittest.TextTestRunner(verbosity=2).run(suite)



import unittest
import os
import numpy as np
from phonopy.structure.grid_points import GridPoints

data_dir = os.path.dirname(os.path.abspath(__file__))


class TestGridPoints(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testGridPoints(self):
        gp = GridPoints([11, 11, 11],
                        [[-1, 1, 1], [1, -1, 1], [1, 1, -1]])
        self.assertTrue((gp.grid_address == gp.get_grid_address()).all())
        self.assertTrue((gp.ir_grid_points == gp.get_ir_grid_points()).all())
        np.testing.assert_allclose(gp.qpoints, gp.get_ir_qpoints())
        self.assertTrue((gp.weights == gp.get_ir_grid_weights()).all())
        self.assertTrue((gp.grid_mapping_table ==
                         gp.get_grid_mapping_table()).all())


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGridPoints)
    unittest.TextTestRunner(verbosity=2).run(suite)



import numpy as np
from phonopy import Phonopy
from phonopy.interface.vasp import read_vasp
from phonopy.file_IO import parse_FORCE_SETS, parse_BORN
import matplotlib.pyplot as plt

def append_band(bands, q_start, q_end):
    band = []
    nq = 51
    for i in range(nq):
        band.append(np.array(q_start) +
                    (np.array(q_end) - np.array(q_start)) / (nq - 1) * i)
    bands.append(band)

unitcell = read_vasp("POSCAR")
phonon = Phonopy(unitcell,
                 [[2, 0, 0],
                  [0, 2, 0],
                  [0, 0, 2]],
                 primitive_matrix=[[0, 0.5, 0.5],
                                   [0.5, 0, 0.5],
                                   [0.5, 0.5, 0]])

force_sets = parse_FORCE_SETS()
phonon.set_displacement_dataset(force_sets)
phonon.produce_force_constants()
primitive = phonon.get_primitive()
nac_params = parse_BORN(primitive, filename="BORN")
phonon.set_nac_params(nac_params)
phonon.set_group_velocity()
bands = []
append_band(bands, [0.0, 0.0, 0.0], [0.5, 0.0, 0.0])
append_band(bands, [0.5, 0.0, 0.0], [0.5, 0.5, 0.0])
append_band(bands, [0.5, 0.5, 0.0], [0.0, 0.0, 0.0])
append_band(bands, [0.0, 0.0, 0.0], [0.5, 0.5, 0.5])
phonon.set_band_structure(bands)
q_points, distances, frequencies, eigvecs = phonon.get_band_structure()
group_velocities = phonon.get_group_velocities_on_bands()
print(len(frequencies))
print(frequencies[0].shape)
print(len(group_velocities))
print(group_velocities[0].shape)



# Copyright (C) 2013 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import warnings
import textwrap
import numpy as np
from phonopy.units import VaspToTHz
from phonopy.harmonic.derivative_dynmat import DerivativeOfDynamicalMatrix
from phonopy.harmonic.force_constants import similarity_transformation
from phonopy.phonon.degeneracy import degenerate_sets


def get_group_velocity(q,  # q-point
                       dynamical_matrix,
                       q_length=None,  # finite distance in q
                       symmetry=None,
                       frequency_factor_to_THz=VaspToTHz):
    """
    If frequencies and eigenvectors are supplied they are used
    instead of calculating them at q-point (but not at q+dq and q-dq).

    reciprocal lattice has to be given as
    [[a_x, b_x, c_x],
     [a_y, b_y, c_y],
     [a_z, b_z, c_z]]
    """

    gv = GroupVelocity(dynamical_matrix,
                       q_length=q_length,
                       symmetry=symmetry,
                       frequency_factor_to_THz=frequency_factor_to_THz)
    gv.set_q_points([q])
    return gv.get_group_velocity()[0]


def delta_dynamical_matrix(q,
                           delta_q,
                           dynmat):
    dynmat.set_dynamical_matrix(q - delta_q)
    dm1 = dynmat.get_dynamical_matrix()
    dynmat.set_dynamical_matrix(q + delta_q)
    dm2 = dynmat.get_dynamical_matrix()
    return dm2 - dm1


class GroupVelocity(object):
    """
    d omega   ----
    ------- = \  / omega
    d q        \/q

    Gradient of omega in reciprocal space.

             d D(q)
    <e(q,nu)|------|e(q,nu)>
              d q
    """

    def __init__(self,
                 dynamical_matrix,
                 q_length=None,
                 symmetry=None,
                 frequency_factor_to_THz=VaspToTHz,
                 cutoff_frequency=1e-4):
        """
        q_points is a list of sets of q-point and q-direction:
        [[q-point, q-direction], [q-point, q-direction], ...]

        q_length is used such as D(q + q_length) - D(q - q_length).
        """
        self._dynmat = dynamical_matrix
        primitive = dynamical_matrix.get_primitive()
        self._reciprocal_lattice_inv = primitive.get_cell()
        self._reciprocal_lattice = np.linalg.inv(self._reciprocal_lattice_inv)
        self._q_length = q_length
        if self._dynmat.is_nac() and self._dynmat.get_nac_method() == 'gonze':
            if self._q_length is None:
                self._q_length = 1e-5
        if self._q_length is None:
            self._ddm = DerivativeOfDynamicalMatrix(dynamical_matrix)
        else:
            self._ddm = None
        self._symmetry = symmetry
        self._factor = frequency_factor_to_THz
        self._cutoff_frequency = cutoff_frequency

        self._directions = np.array([[1, 2, 3],
                                     [1, 0, 0],
                                     [0, 1, 0],
                                     [0, 0, 1]], dtype='double')
        self._directions[0] /= np.linalg.norm(self._directions[0])

        self._q_points = None
        self._group_velocity = None
        self._perturbation = None

    def set_q_points(self, q_points, perturbation=None):
        self._q_points = q_points
        self._perturbation = perturbation
        if perturbation is None:
            self._directions[0] = np.array([1, 2, 3])
        else:
            self._directions[0] = np.dot(
                self._reciprocal_lattice, perturbation)
        self._directions[0] /= np.linalg.norm(self._directions[0])
        self._set_group_velocity()

    def set_q_length(self, q_length):
        self._q_length = q_length

    def get_q_length(self):
        return self._q_length

    def get_group_velocity(self):
        return self._group_velocity

    def _set_group_velocity(self):
        gv = [self._set_group_velocity_at_q(q) for q in self._q_points]
        self._group_velocity = np.array(gv)

    def _set_group_velocity_at_q(self, q):
        self._dynmat.set_dynamical_matrix(q)
        dm = self._dynmat.get_dynamical_matrix()
        eigvals, eigvecs = np.linalg.eigh(dm)
        eigvals = eigvals.real
        freqs = np.sqrt(abs(eigvals)) * np.sign(eigvals) * self._factor
        gv = np.zeros((len(freqs), 3), dtype='double')
        deg_sets = degenerate_sets(freqs)

        ddms = self._get_dD(np.array(q))
        pos = 0
        for deg in deg_sets:
            gv[pos:pos+len(deg)] = self._perturb_D(ddms, eigvecs[:, deg])
            pos += len(deg)

        for i, f in enumerate(freqs):
            if f > self._cutoff_frequency:
                gv[i, :] *= self._factor ** 2 / f / 2
            else:
                gv[i, :] = 0

        if self._perturbation is None:
            return self._symmetrize_group_velocity(gv, q)
        else:
            return gv

    def _symmetrize_group_velocity(self, gv, q):
        rotations = []
        for r in self._symmetry.get_reciprocal_operations():
            q_in_BZ = q - np.rint(q)
            diff = q_in_BZ - np.dot(r, q_in_BZ)
            if (np.abs(diff) < self._symmetry.get_symmetry_tolerance()).all():
                rotations.append(r)

        gv_sym = np.zeros_like(gv)
        for r in rotations:
            r_cart = similarity_transformation(self._reciprocal_lattice, r)
            gv_sym += np.dot(r_cart, gv.T).T

        return gv_sym / len(rotations)

    def _get_dD(self, q):
        if self._q_length is None:
            return self._get_dD_analytical(q)
        else:
            return self._get_dD_FD(q)

    def _get_dD_FD(self, q): # finite difference
        ddm = []
        for dqc in self._directions * self._q_length:
            dq = np.dot(self._reciprocal_lattice_inv, dqc)
            ddm.append(delta_dynamical_matrix(q, dq, self._dynmat) /
                       self._q_length / 2)
        return np.array(ddm)

    def _get_dD_analytical(self, q):
        self._ddm.run(q)
        ddm = self._ddm.get_derivative_of_dynamical_matrix()
        dtype = "c%d" % (np.dtype('double').itemsize * 2)
        ddm_dirs = np.zeros((len(self._directions),) + ddm.shape[1:],
                            dtype=dtype)
        for i, dq in enumerate(self._directions):
            for j in range(3):
                ddm_dirs[i] += dq[j] * ddm[j]
        return ddm_dirs

    def _perturb_D(self, ddms, eigsets):
        eigvals, eigvecs = np.linalg.eigh(
            np.dot(eigsets.T.conj(), np.dot(ddms[0], eigsets)))

        gv = []
        rot_eigsets = np.dot(eigsets, eigvecs)
        for ddm in ddms[1:]:
            gv.append(
                np.diag(np.dot(rot_eigsets.T.conj(),
                               np.dot(ddm, rot_eigsets))).real)

        return np.transpose(gv)



# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np
import phonopy.structure.spglib as spg
from phonopy.structure.cells import (get_primitive, get_supercell,
                                     compute_all_sg_permutations)
from phonopy.structure.atoms import PhonopyAtoms as Atoms
from phonopy.harmonic.force_constants import similarity_transformation


class Symmetry(object):
    def __init__(self, cell, symprec=1e-5, is_symmetry=True):
        self._cell = cell
        self._symprec = symprec

        self._symmetry_operations = None
        self._international_table = None
        self._dataset = None
        self._wyckoff_letters = None
        self._map_atoms = None
        self._atomic_permutations = None

        magmom = cell.get_magnetic_moments()
        if type(magmom) is np.ndarray:
            if (magmom < symprec).all():
                magmom = None

        if not is_symmetry:
            self._set_nosym()
        elif magmom is None:
            self._set_symmetry_dataset()
        else:
            self._set_symmetry_operations_with_magmoms()

        self._pointgroup_operations = None
        self._pointgroup = None
        self._set_pointgroup_operations()

        self._independent_atoms = None
        self._set_independent_atoms()
        self._map_operations = None

    def get_symmetry_operations(self):
        return self._symmetry_operations

    def get_symmetry_operation(self, operation_number):
        operation = self._symmetry_operations
        return {'rotations': operation['rotations'][operation_number],
                'translations': operation['translations'][operation_number]}

    def get_pointgroup_operations(self):
        return self._pointgroup_operations

    def get_pointgroup(self):
        return self._pointgroup

    def get_international_table(self):
        return self._international_table

    def get_Wyckoff_letters(self):
        return self._wyckoff_letters

    @property
    def dataset(self):
        return self._dataset

    def get_dataset(self):
        return self.dataset

    def get_independent_atoms(self):
        return self._independent_atoms

    def get_map_atoms(self):
        return self._map_atoms

    def get_map_operations(self):
        if self._map_operations is None:
            self._set_map_operations()
        return self._map_operations

    def get_site_symmetry(self, atom_number):
        positions = self._cell.get_scaled_positions()
        lattice = self._cell.get_cell()
        rotations = self._symmetry_operations['rotations']
        translations = self._symmetry_operations['translations']

        return self._get_site_symmetry(atom_number,
                                       lattice,
                                       positions,
                                       rotations,
                                       translations,
                                       self._symprec)

    def get_symmetry_tolerance(self):
        return self._symprec

    def get_reciprocal_operations(self):
        """
        Definition of operation:
        q' = Rq

        This is transpose of that shown in ITA (q' = qR).
        """
        return self._reciprocal_operations

    def get_atomic_permutations(self):
        if self._atomic_permutations is None:
            positions = self._cell.get_scaled_positions()
            lattice = np.array(self._cell.get_cell().T,
                               dtype='double', order='C')
            rotations = self._symmetry_operations['rotations']
            translations = self._symmetry_operations['translations']
            self._atomic_permutations = compute_all_sg_permutations(
                positions,  # scaled positions
                rotations,  # scaled
                translations,  # scaled
                lattice,  # column vectors
                self._symprec)

        return self._atomic_permutations

    def _get_pointgroup_operations(self, rotations):
        ptg_ops = []
        for rot in rotations:
            is_same = False
            for tmp_rot in ptg_ops:
                if (tmp_rot == rot).all():
                    is_same = True
                    break
            if not is_same:
                ptg_ops.append(rot)

        return ptg_ops

    def _get_site_symmetry(self,
                           atom_number,
                           lattice,
                           positions,
                           rotations,
                           translations,
                           symprec):
        pos = positions[atom_number]
        site_symmetries = []

        for r, t in zip(rotations, translations):
            rot_pos = np.dot(pos, r.T) + t
            diff = pos - rot_pos
            diff -= np.rint(diff)
            diff = np.dot(diff, lattice)
            if np.linalg.norm(diff) < symprec:
                site_symmetries.append(r)

        return np.array(site_symmetries, dtype='intc')

    def _set_symmetry_dataset(self):
        self._dataset = spg.get_symmetry_dataset(self._cell.totuple(),
                                                 self._symprec)
        self._symmetry_operations = {
            'rotations': self._dataset['rotations'],
            'translations': self._dataset['translations']}
        self._international_table = "%s (%d)" % (
            self._dataset['international'], self._dataset['number'])
        self._wyckoff_letters = self._dataset['wyckoffs']

        self._map_atoms = self._dataset['equivalent_atoms']

    def _set_symmetry_operations_with_magmoms(self):
        self._symmetry_operations = spg.get_symmetry(self._cell.totuple(),
                                                     symprec=self._symprec)
        self._map_atoms = self._symmetry_operations['equivalent_atoms']
        self._set_map_atoms()

    def _set_map_atoms(self):
        rotations = self._symmetry_operations['rotations']
        translations = self._symmetry_operations['translations']
        positions = self._cell.get_scaled_positions()
        lattice = self._cell.get_cell()
        map_atoms = np.arange(self._cell.get_number_of_atoms())
        for i, p in enumerate(positions):
            is_found = False
            for j in range(i):
                for r, t in zip(rotations, translations):
                    diff = np.dot(p, r.T) + t - positions[j]
                    diff -= np.rint(diff)
                    dist = np.linalg.norm(np.dot(diff, lattice))
                    if dist < self._symprec:
                        map_atoms[i] = j
                        is_found = True
                        break
                if is_found:
                    break
        self._map_atoms = np.array(map_atoms, dtype='intc')

    def _set_independent_atoms(self):
        indep_atoms = []
        for i, atom_map in enumerate(self._map_atoms):
            if i == atom_map:
                indep_atoms.append(i)
        self._independent_atoms = np.array(indep_atoms, dtype='intc')

    def _set_pointgroup_operations(self):
        rotations = self._symmetry_operations['rotations']
        ptg_ops = self._get_pointgroup_operations(rotations)
        reciprocal_rotations = [rot.T for rot in ptg_ops]
        exist_r_inv = False
        for rot in ptg_ops:
            if (rot + np.eye(3, dtype='intc') == 0).all():
                exist_r_inv = True
                break
        if not exist_r_inv:
            reciprocal_rotations += [-rot.T for rot in ptg_ops]

        self._pointgroup_operations = np.array(ptg_ops, dtype='intc')
        self._pointgroup = get_pointgroup(self._pointgroup_operations)[0]
        self._reciprocal_operations = np.array(reciprocal_rotations,
                                               dtype='intc')

    def _set_map_operations(self):
        ops = self._symmetry_operations
        pos = self._cell.get_scaled_positions()
        lattice = self._cell.get_cell()
        map_operations = np.zeros(len(pos), dtype='intc')

        for i, eq_atom in enumerate(self._map_atoms):
            for j, (r, t) in enumerate(zip(ops['rotations'],
                                           ops['translations'])):
                diff = np.dot(pos[i], r.T) + t - pos[eq_atom]
                diff -= np.rint(diff)
                dist = np.linalg.norm(np.dot(diff, lattice))
                if dist < self._symprec:
                    map_operations[i] = j
                    break
        self._map_operations = map_operations

    def _set_nosym(self):
        translations = []
        rotations = []

        if 'get_supercell_to_unitcell_map' in dir(self._cell):
            s2u_map = self._cell.get_supercell_to_unitcell_map()
            positions = self._cell.get_scaled_positions()

            for i, j in enumerate(s2u_map):
                if j==0:
                    ipos0 = i
                    break

            for i, p in zip(s2u_map, positions):
                if i==0:
                    trans = p - positions[ipos0]
                    trans -= np.floor(trans)
                    translations.append(trans)
                    rotations.append(np.eye(3, dtype='intc'))

            self._map_atoms = s2u_map
        else:
            rotations.append(np.eye(3, dtype='intc'))
            translations.append(np.zeros(3, dtype='double'))
            self._map_atoms = range(self._cell.get_number_of_atoms())

        self._symmetry_operations = {
            'rotations': np.array(rotations, dtype='intc'),
            'translations': np.array(translations, dtype='double')}
        self._international_table = 'P1 (1)'
        self._wyckoff_letters = ['a'] * self._cell.get_number_of_atoms()


def find_primitive(cell, symprec=1e-5):
    """
    A primitive cell is searched in the input cell. When a primitive
    cell is found, an object of Atoms class of the primitive cell is
    returned. When not, None is returned.
    """
    lattice, positions, numbers = spg.find_primitive(cell.totuple(), symprec)
    if lattice is None:
        return None
    else:
        return Atoms(numbers=numbers,
                     scaled_positions=positions,
                     cell=lattice,
                     pbc=True)


def get_pointgroup(rotations):
    ptg = spg.get_pointgroup(rotations)
    return ptg[0].strip(), ptg[2]


def get_lattice_vector_equivalence(point_symmetry):
    """Return (b==c, c==a, a==b)"""
    # primitive_vectors: column vectors

    equivalence = [False, False, False]
    for r in point_symmetry:
        if (np.abs(r[:, 0]) == [0, 1, 0]).all():
            equivalence[2] = True
        if (np.abs(r[:, 0]) == [0, 0, 1]).all():
            equivalence[1] = True
        if (np.abs(r[:, 1]) == [1, 0, 0]).all():
            equivalence[2] = True
        if (np.abs(r[:, 1]) == [0, 0, 1]).all():
            equivalence[0] = True
        if (np.abs(r[:, 2]) == [1, 0, 0]).all():
            equivalence[1] = True
        if (np.abs(r[:, 2]) == [0, 1, 0]).all():
            equivalence[0] = True

    return equivalence


def elaborate_borns_and_epsilon(ucell,
                                borns,
                                epsilon,
                                primitive_matrix=None,
                                supercell_matrix=None,
                                is_symmetry=True,
                                symmetrize_tensors=False,
                                symprec=1e-5):
    """Symmetrize Born effective charges and dielectric constants and
    extract Born effective charges of symmetrically independent atoms
    for primitive cell.


     Args:
         ucell (Atoms): Unit cell structure
         borns (np.array): Born effective charges of ucell
         epsilon (np.array): Dielectric constant tensor

     Returns:
         (np.array) Born effective charges of symmetrically independent atoms
             in primitive cell
         (np.array) Dielectric constant
         (np.array) Atomic index mapping table from supercell to primitive cell
             of independent atoms

     Raises:
          AssertionError: Inconsistency of number of atoms or Born effective
              charges.

     Warning:
         Broken symmetry of Born effective charges

    """

    assert len(borns) == ucell.get_number_of_atoms(), \
        "num_atom %d != len(borns) %d" % (ucell.get_number_of_atoms(),
                                          len(borns))

    if symmetrize_tensors:
        borns_, epsilon_ = symmetrize_borns_and_epsilon(
            borns,
            epsilon,
            ucell,
            symprec=symprec,
            is_symmetry=is_symmetry)
    else:
        borns_ = borns
        epsilon_ = epsilon

    indeps_in_supercell, indeps_in_unitcell = _extract_independent_atoms(
        ucell,
        primitive_matrix=primitive_matrix,
        supercell_matrix=supercell_matrix,
        is_symmetry=is_symmetry,
        symprec=symprec)

    return borns_[indeps_in_unitcell].copy(), epsilon_, indeps_in_supercell


def symmetrize_borns_and_epsilon(borns,
                                 epsilon,
                                 ucell,
                                 primitive_matrix=None,
                                 primitive=None,
                                 supercell_matrix=None,
                                 symprec=1e-5,
                                 is_symmetry=True):
    """Symmetrize Born effective charges and dielectric tensor

    Parameters
    ----------
    borns: array_like
        Born effective charges.
        shape=(unitcell_atoms, 3, 3)
        dtype='double'
    epsilon: array_like
        Dielectric constant
        shape=(3, 3)
        dtype='double'
    ucell: PhonopyAtoms
        Unit cell
    primitive_matrix: array_like, optional
        Primitive matrix. This is used to select Born effective charges in
        primitive cell. If None (default), Born effective charges in unit cell
        are returned.
        shape=(3, 3)
        dtype='double'
    primitive : PhonopyAtoms
        This is an alternative of giving primitive_matrix (Mp). Mp is given as
            Mp = (a_u, b_u, c_u)^-1 * (a_p, b_p, c_p).
        In addition, the order of atoms is alined to those of atoms in this
        primitive cell for Born effective charges. No rigid rotation of
        crystal structure is assumed.
    supercell_matrix: array_like, optional
        Supercell matrix. This is used to select Born effective charges in
        **primitive cell**. Supercell matrix is needed because primitive
        cell is created first creating supercell from unit cell, then
        the primitive cell is created from the supercell. If None (defautl),
        1x1x1 supercell is created.
        shape=(3, 3)
        dtype='int'
    symprec: float, optional
        Symmetry tolerance. Default is 1e-5
    is_symmetry: bool, optinal
        By setting False, symmetrization can be switched off. Default is True.

    """

    lattice = ucell.get_cell()
    positions = ucell.get_scaled_positions()
    u_sym = Symmetry(ucell, is_symmetry=is_symmetry, symprec=symprec)
    rotations = u_sym.get_symmetry_operations()['rotations']
    translations = u_sym.get_symmetry_operations()['translations']
    ptg_ops = u_sym.get_pointgroup_operations()
    epsilon_ = _symmetrize_2nd_rank_tensor(epsilon, ptg_ops, lattice)

    for i, Z in enumerate(borns):
        site_sym = u_sym.get_site_symmetry(i)
        Z = _symmetrize_2nd_rank_tensor(Z, site_sym, lattice)

    borns_ = np.zeros_like(borns)
    for i in range(len(borns)):
        count = 0
        for r, t in zip(rotations, translations):
            count += 1
            diff = np.dot(positions, r.T) + t - positions[i]
            diff -= np.rint(diff)
            dist = np.sqrt(np.sum(np.dot(diff, lattice) ** 2, axis=1))
            j = np.nonzero(dist < symprec)[0][0]
            r_cart = similarity_transformation(lattice.T, r)
            borns_[i] += similarity_transformation(r_cart, borns[j])
        borns_[i] /= count

    sum_born = borns_.sum(axis=0) / len(borns_)
    borns_ -= sum_born

    if (abs(borns - borns_) > 0.1).any():
        lines = ["Born effective charge symmetry is largely broken. "
                 "Largest different among elements: "
                 "%s" % np.amax(abs(borns - borns_))]
        import warnings
        warnings.warn("\n".join(lines))

    if primitive_matrix is None and primitive is None:
        return borns_, epsilon_
    else:
        if primitive is not None:
            pmat = np.dot(np.linalg.inv(ucell.cell.T), primitive.cell.T)
        else:
            pmat = primitive_matrix

        scell, pcell = _get_supercell_and_primitive(
            ucell,
            primitive_matrix=pmat,
            supercell_matrix=supercell_matrix,
            symprec=symprec)

        idx = [scell.u2u_map[i] for i in scell.s2u_map[pcell.p2s_map]]
        borns_in_prim = borns_[idx].copy()

        if primitive is None:
            return borns_in_prim, epsilon_
        else:
            idx2 = _get_mapping_between_cells(pcell, primitive)
            return borns_in_prim[idx2].copy(), epsilon_


def _get_mapping_between_cells(cell_from, cell_to, symprec=1e-5):
    indices = []
    lattice = cell_from.cell
    pos_from = cell_from.scaled_positions
    for i, p_to in enumerate(cell_to.scaled_positions):
        diff = pos_from - p_to
        diff -= np.rint(diff)
        dist = np.sqrt(np.sum(np.dot(diff, lattice) ** 2, axis=1))
        ids = np.nonzero(dist < symprec)[0]
        if len(ids) == 1:
            indices.append(ids[0])
        else:
            msg = "Index matching didn't go well."
            raise RuntimeError(msg)
    return indices


def _symmetrize_2nd_rank_tensor(tensor, symmetry_operations, lattice):
    sym_cart = [similarity_transformation(lattice.T, r)
                for r in symmetry_operations]
    sum_tensor = np.zeros_like(tensor)
    for sym in sym_cart:
        sum_tensor += similarity_transformation(sym, tensor)
    return sum_tensor / len(symmetry_operations)


def _extract_independent_atoms(ucell,
                               primitive_matrix=None,
                               supercell_matrix=None,
                               is_symmetry=True,
                               symprec=1e-5):
    scell, pcell = _get_supercell_and_primitive(
        ucell,
        primitive_matrix=primitive_matrix,
        supercell_matrix=supercell_matrix,
        symprec=symprec)
    p_sym = Symmetry(pcell, is_symmetry=is_symmetry, symprec=symprec)
    s_indep_atoms = pcell.p2s_map[p_sym.get_independent_atoms()]
    u_indep_atoms = [scell.u2u_map[x] for x in s_indep_atoms]

    return s_indep_atoms, u_indep_atoms


def _get_supercell_and_primitive(ucell,
                                 primitive_matrix=None,
                                 supercell_matrix=None,
                                 symprec=1e-5):
    if primitive_matrix is None:
        pmat = np.eye(3)
    else:
        pmat = primitive_matrix
    if supercell_matrix is None:
        smat = np.eye(3, dtype='intc')
    else:
        smat = supercell_matrix

    inv_smat = np.linalg.inv(smat)
    scell = get_supercell(ucell, smat, symprec=symprec)
    pcell = get_primitive(scell, np.dot(inv_smat, pmat), symprec=symprec)

    return scell, pcell



#!/usr/bin/env python

from phonopy import Phonopy
from phonopy.interface.vasp import read_vasp
from phonopy.file_IO import parse_FORCE_SETS, parse_BORN
import yaml
import numpy as np

unitcell = read_vasp("POSCAR")
phonon = Phonopy(unitcell,
                 [[2, 0, 0],
                  [0, 2, 0],
                  [0, 0, 2]],
                 primitive_matrix=[[0, 0.5, 0.5],
                                   [0.5, 0, 0.5],
                                   [0.5, 0.5, 0]])
force_sets = parse_FORCE_SETS()
phonon.set_displacement_dataset(force_sets)
phonon.produce_force_constants()

q = [0.1, 0.1, 0.1]
dynmat = phonon.get_dynamical_matrix_at_q(q)
print(dynmat)
phonon.set_qpoints_phonon(q, write_dynamical_matrices=True)
print(phonon.get_qpoints_phonon()[0][0])
phonon.write_yaml_qpoints_phonon()

data = yaml.load(open("qpoints.yaml"))
dynmat_from_yaml = []
dynmat_data = data['phonon'][0]['dynamical_matrix']
for row in dynmat_data:
    vals = np.reshape(row, (-1, 2))
    dynmat_from_yaml.append(vals[:, 0] + vals[:, 1] * 1j)
dynmat_from_yaml = np.array(dynmat_from_yaml)
print(dynmat_from_yaml)
eigvals, eigvecs, = np.linalg.eigh(dynmat)
frequencies = np.sqrt(np.abs(eigvals.real)) * np.sign(eigvals.real)
conversion_factor_to_THz = 15.633302
print(frequencies * conversion_factor_to_THz)





import unittest
import numpy as np
from phonopy import Phonopy
from phonopy.interface.vasp import read_vasp
from phonopy.file_IO import parse_FORCE_SETS
import os

data_dir = os.path.dirname(os.path.abspath(__file__))


class TestDynamicalMatrix(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_properties(self):
        cell = read_vasp(os.path.join(data_dir, "..", "POSCAR_NaCl"))
        phonon = Phonopy(cell,
                         np.diag([2, 2, 2]),
                         primitive_matrix=[[0, 0.5, 0.5],
                                           [0.5, 0, 0.5],
                                           [0.5, 0.5, 0]])
        filename = os.path.join(data_dir, "..", "FORCE_SETS_NaCl")
        force_sets = parse_FORCE_SETS(filename=filename)
        phonon.set_displacement_dataset(force_sets)
        phonon.produce_force_constants()
        dynmat = phonon.dynamical_matrix
        dynmat.set_dynamical_matrix([0, 0, 0])
        self.assertTrue(id(dynmat.primitive)
                        == id(dynmat.get_primitive()))
        self.assertTrue(id(dynmat.supercell)
                        == id(dynmat.get_supercell()))
        np.testing.assert_allclose(dynmat.dynamical_matrix,
                                   dynmat.get_dynamical_matrix())


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestDynamicalMatrix)
    unittest.TextTestRunner(verbosity=2).run(suite)



from phonopy import Phonopy
from phonopy.interface.vasp import read_vasp
from phonopy.file_IO import parse_FORCE_SETS, parse_BORN
import numpy as np

cell = read_vasp("POSCAR")

# Initialize phonon. Supercell matrix has to have the shape of (3, 3)
phonon = Phonopy(cell,
                 np.diag([2, 2, 1]),
                 primitive_matrix=[[2./3, -1./3, -1./3],
                                   [1./3, 1./3, -2./3],
                                   [1./3, 1./3, 1./3]])

symmetry = phonon.get_symmetry()
print("Space group: %s" % symmetry.get_international_table())

force_sets = parse_FORCE_SETS()
phonon.set_displacement_dataset(force_sets)
phonon.produce_force_constants()

born = parse_BORN(phonon.get_primitive())
phonon.set_nac_params(born)

# Example to obtain dynamical matrix
dmat = phonon.get_dynamical_matrix_at_q([0,0,0])
print(dmat)

# Example of band structure calculation
bands = []
q_start = np.array([1./3, 1./3, 0])
q_end = np.array([0, 0, 0])
band = []
for i in range(51):
    band.append(q_start + (q_end - q_start) / 50 * i)
bands.append(band)

q_start = np.array([0, 0, 0])
q_end = np.array([1./3, 1./3, 1./2])
band = []
for i in range(51):
    band.append(q_start + ( q_end - q_start ) / 50 * i)
bands.append(band)

#*********************
# Matplotlib required
#*********************
print("\nPhonon dispersion:")
phonon.set_band_structure(bands,
                          is_eigenvectors=True)
band_plot = phonon.plot_band_structure(["X", "$\Gamma$", "L"])
band_plot.show()

bands = phonon.get_band_structure()
distances = bands[1]
frequencies = bands[2]
qpoints = bands[0]

for (qs_at_segments,
     dists_at_segments,
     freqs_at_segments) in zip(qpoints, distances, frequencies):

    for q, d, f in zip(qs_at_segments,
                       dists_at_segments,
                       freqs_at_segments):
        print("# %f %f %f" % tuple(q))
        print(("%s " + "%f " * len(f)) % ((d,) + tuple(f)))

# If you just want to plot along q-points of all band segments, the
# following is easier.
        
# all_freqs = np.vstack(frequencies)
# all_qs = np.vstack(qpoints)
# all_dists = np.hstack(distances)
# all_eigvecs = np.concatenate(eigvecs)
# print("# shape of eigvecs %s" % all_eigvecs.shape)
# for d, q, f in zip(all_dists, all_qs, all_freqs):
#     print(("%f " * (4 + len(f))) % ((d,) + tuple(q) + tuple(f)))



# Copyright (C) 2016 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys


def fix_deprecated_option_names(argv):
    deprecated = []
    for i, v in enumerate(argv[1:]):
        if v[0] == '-':
            tag = v.split('=')[0]
            if '_' in tag:
                correct_tag = tag.replace('_', '-')
                deprecated.append(tag)
                argv[i + 1] = v.replace(tag, correct_tag)

    return deprecated


def show_deprecated_option_warnings(deprecated):
    lines = ["Option names with underscores are deprecated, by which",
             "the underscores are replaced by dashes. Therefore"]
    for tag in deprecated:
        lines.append("'%s' has to be written as '%s'." %
                     (tag, tag.replace('_', '-')))
    maxlen = max([len(line) for line in lines])
    print("*" * maxlen)
    print('\n'.join(lines))
    print("*" * maxlen)
    print("")


def get_parser():
    deprecated = fix_deprecated_option_names(sys.argv)
    import argparse
    parser = argparse.ArgumentParser(
        description="Phonopy command-line-tool")
    parser.set_defaults(
        abinit_mode=False,
        anime=None,
        band_format=None,
        band_indices=None,
        band_labels=None,
        band_paths=None,
        band_points=None,
        cell_filename=None,
        crystal_mode=False,
        cutoff_frequency=None,
        cutoff_radius=None,
        displacement_distance=None,
        dynamical_matrix_decimals=None,
        elk_mode=False,
        siesta_mode=False,
        cp2k_mode=False,
        dftbp_mode=False,
        fc_calculator_options=None,
        fc_symmetry=None,
        fc_format=None,
        fc_spg_symmetry=False,
        fits_debye_model=False,
        force_constants_decimals=None,
        force_constants=None,
        force_sets=None,
        force_sets_zero=None,
        fmax=None,
        fmin=None,
        frequency_conversion_factor=None,
        fpitch=None,
        frequency_scale_factor=None,
        gv_delta_q=None,
        hdf5_compression="gzip",
        is_band_connection=False,
        is_band_const_interval=False,
        is_check_symmetry=False,
        is_displacement=False,
        is_dos_mode=False,
        is_eigenvectors=False,
        is_full_fc=False,
        is_gamma_center=False,
        is_graph_plot=False,
        is_graph_save=False,
        is_group_velocity=False,
        is_hdf5=False,
        is_legend=False,
        is_legacy_plot=False,
        is_little_cogroup=False,
        is_moment=False,
        is_nac=False,
        is_nodiag=False,
        is_nomeshsym=False,
        is_nosym=False,
        is_plusminus_displacements=False,
        is_tetrahedron_method=True,
        is_thermal_displacements=False,
        is_thermal_displacement_matrices=False,
        is_thermal_displacement_matrices_cif=None,
        is_thermal_properties=False,
        is_projected_thermal_properties=False,
        is_trigonal_displacements=False,
        is_wien2k_p1=False,
        irreps_qpoint=None,
        lapack_solver=False,
        loglevel=None,
        masses=None,
        magmom=None,
        mesh_format=None,
        mesh_numbers=None,
        modulation=None,
        moment_order=None,
        nac_method=None,
        nac_q_direction=None,
        pdos=None,
        pretend_real=False,
        primitive_axes=None,
        projection_direction=None,
        qe_mode=False,
        qpoints=None,
        qpoints_format=None,
        quiet=False,
        random_displacemen=None,
        random_seed=None,
        read_fc_format=None,
        read_force_constants=False,
        read_qpoints=False,
        show_irreps=False,
        sigma=None,
        supercell_dimension=None,
        symmetry_tolerance=None,
        temperature=None,
        tmax=None,
        tmin=None,
        tstep=None,
        turbomooe_mode=False,
        use_alm=False,
        use_hiphive=False,
        vasp_mode=False,
        verbose=False,
        wien2k_mode=False,
        write_dynamical_matrices=False,
        write_fc_format=None,
        write_force_constants=False,
        write_mesh=True)

    parser.add_argument(
        "--abinit", dest="abinit_mode", action="store_true",
        help="Invoke Abinit mode")
    parser.add_argument(
        "--alm", dest="use_alm", action="store_true",
        help="Use ALM for generating force constants")
    parser.add_argument(
        "--amplitude", dest="displacement_distance", type=float,
        help="Distance of displacements")
    parser.add_argument(
        "--anime", nargs='+', dest="anime",
        help="Same as ANIME tag")
    parser.add_argument(
        "--band", nargs='+', dest="band_paths",
        help="Same behavior as BAND tag")
    parser.add_argument(
        "--band-connection", dest="is_band_connection", action="store_true",
        help="Treat band crossings")
    parser.add_argument(
        "--band-const-interval", dest="is_band_const_interval",
        action="store_true",
        help="Band paths are sampled with similar interval.")
    parser.add_argument(
        "--band-labels", nargs='+', dest="band_labels",
        help="Show labels at band segments")
    parser.add_argument(
        "--band-format", dest="band_format",
        help="Band structure output file-format")
    parser.add_argument(
        "--band-points", dest="band_points", type=int,
        help=("Number of points calculated on a band segment in "
              "the band structure mode"))
    parser.add_argument(
        "--bi", "--band-indices", nargs='+', dest="band_indices",
        help=("Band indices to be included to calcualte thermal "
              "properties"))
    parser.add_argument(
        "-c", "--cell", dest="cell_filename", metavar="FILE",
        help="Read unit cell")
    parser.add_argument(
        "--cp2k", dest="cp2k_mode", action="store_true",
        help="Invoke CP2K mode")
    parser.add_argument(
        "--crystal", dest="crystal_mode", action="store_true",
        help="Invoke CRYSTAL mode")
    parser.add_argument(
        "--cutoff-freq", "--cutoff-frequency", dest="cutoff_frequency",
        type=float,
        help=("Thermal properties are not calculated below this "
              "cutoff frequency."))
    parser.add_argument(
        "--cutoff-radius", dest="cutoff_radius", type=float,
        help="Out of cutoff radius, force constants are set zero.")
    parser.add_argument(
        "-d", "--displacement", dest="is_displacement", action="store_true",
        help="Create supercells with displacements")
    parser.add_argument(
        "--dftb+", dest="dftbp_mode", action="store_true",
        help="Invoke dftb+ mode")
    parser.add_argument(
        "--dim", nargs='+', dest="supercell_dimension",
        help="Same behavior as DIM tag")
    parser.add_argument(
        "--dm-decimals", dest="dynamical_matrix_decimals",
        type=int, help="Decimals of values of decimals")
    parser.add_argument(
        "--dos", dest="is_dos_mode", action="store_true",
        help="Calculate (P)DOS")
    parser.add_argument(
        "--eigvecs", "--eigenvectors", dest="is_eigenvectors",
        action="store_true",
        help="Output eigenvectors")
    parser.add_argument(
        "--elk", dest="elk_mode", action="store_true",
        help="Invoke elk mode")
    parser.add_argument(
        "-f", "--force-sets", nargs='+', dest="force_sets",
        help="Create FORCE_SETS")
    parser.add_argument(
        "--factor", dest="frequency_conversion_factor", type=float,
        help="Frequency unit conversion factor")
    parser.add_argument(
        "--fc", "--force-constants", nargs=1, dest="force_constants",
        help=("Create FORCE_CONSTANTS from vaspurn.xml. "
              "vasprun.xml has to be passed as argument."))
    parser.add_argument(
        "--fc-calc-opt", "--fc-calculator-options",
        dest="fc_calculator_options",
        help=("Options for force constants calculator as comma separated "
              "string with the style of key = values"))
    parser.add_argument(
        "--fc-decimals", dest="force_constants_decimals", type=int,
        help="Decimals of values of force constants")
    parser.add_argument(
        "--fc-format", dest="fc_format",
        help="Force constants input/output file-format")
    parser.add_argument(
        "--fc-spg-symmetry", dest="fc_spg_symmetry", action="store_true",
        help="Enforce space group symmetry to force constants")
    parser.add_argument(
        "--fc-symmetry", dest="fc_symmetry", action="store_true",
        help="Symmetrize force constants")
    parser.add_argument(
        "--fits-debye-model", dest="fits_debye_model", action="store_true",
        help="Fits total DOS to a Debye model")
    parser.add_argument(
        "--freq-scale", dest="frequency_scale_factor", type=float,
        help=("Squared scale factor multiplied as fc2 * factor^2. Therefore "
              "frequency is changed but the contribution from NAC is not "
              "changed."))
    parser.add_argument(
        "--full-fc", dest="is_full_fc", action="store_true",
        help="Calculate full supercell force constants matrix")
    parser.add_argument(
        "--fz", "--force-sets-zero", nargs='+', dest="force_sets_zero",
        help=("Create FORCE_SETS. disp.yaml in the current directory and "
              "vapsrun.xml's for VASP or case.scf(m) for Wien2k as arguments "
              "are required. The first argument is that of the perfect "
              "supercell to subtract residual forces"))
    parser.add_argument(
        "--fmax", dest="fmax", type=float,
        help="Maximum frequency used for DOS or moment calculation")
    parser.add_argument(
        "--fmin", dest="fmin", type=float,
        help="Minimum frequency used for DOS or moment calculation")
    parser.add_argument(
        "--fpitch", dest="fpitch", type=float,
        help="Frequency pitch used for DOS or moment calculation")
    parser.add_argument(
        "--gc", "--gamma-center", dest="is_gamma_center", action="store_true",
        help="Set mesh as Gamma center")
    parser.add_argument(
        "--gv", "--group-velocity", dest="is_group_velocity",
        action="store_true",
        help="Calculate group velocities at q-points")
    parser.add_argument(
        "--gv-delta-q", dest="gv_delta_q", type=float,
        help="Delta-q distance used for group velocity calculation")
    parser.add_argument(
        "--hdf5", dest="is_hdf5", action="store_true",
        help="Use hdf5 for force constants")
    parser.add_argument(
        "--hdf5-compression", dest="hdf5_compression",
        help="hdf5 compression filter")
    parser.add_argument(
        "--hiphive", dest="use_hiphive", action="store_true",
        help="Use hiPhive for generating force constants")
    parser.add_argument(
        "--irreps", "--irreps-qpoint", nargs='+', dest="irreps_qpoint",
        help="A q-point where characters of irreps are calculated")
    # parser.add_argument(
    #     "--lapack-solver", dest="lapack_solver", action="store_true",
    #     help=("Use Lapack via Lapacke for solving phonons. This "
    #           "option can be used only when phonopy is compiled "
    #           "specially."))
    parser.add_argument(
        "--legend", dest="is_legend", action="store_true",
        help="Legend of plots is shown in thermal displacements")
    parser.add_argument(
        "--legacy-plot", dest="is_legacy_plot", action="store_true",
        help="Legacy style band structure pl")
    parser.add_argument(
        "--lcg", "--little-cogroup", dest="is_little_cogroup",
        action="store_true",
        help=("Show irreps of little co-group (or point-group of "
              "wave vector q) instead of little group"))
    parser.add_argument(
        "--loglevel", dest="loglevel", type=int,
        help="Log level")
    parser.add_argument(
        "--mass", nargs='+', dest="masses",
        help="Same as MASS tag")
    parser.add_argument(
        "--magmom", nargs='+', dest="magmoms",
        help="Same as MAGMOM tag")
    parser.add_argument(
        "--mesh-format", dest="mesh_format",
        help="Mesh output file-format")
    parser.add_argument(
        "--modulation", nargs='+', dest="modulation",
        help="Same as MODULATION tag")
    parser.add_argument(
        "--mp", "--mesh", nargs='+', dest="mesh_numbers",
        help="Same behavior as MP tag")
    parser.add_argument(
        "--moment", dest="is_moment", action="store_true",
        help="Calculate moment of phonon states distribution")
    parser.add_argument(
        "--moment-order", dest="moment_order",
        type=int, help="Order of moment of phonon states distribution")
    parser.add_argument(
        "--nac", dest="is_nac", action="store_true",
        help="Non-analytical term correction")
    parser.add_argument(
        "--nac-method", dest="nac_method",
        help="Non-analytical term correction method: Gonze (default) or Wang")
    parser.add_argument(
        "--nodiag", dest="is_nodiag", action="store_true",
        help="Set displacements parallel to axes")
    parser.add_argument(
        "--nomeshsym", dest="is_nomeshsym", action="store_true",
        help="Symmetry is not imposed for mesh sampling.")
    parser.add_argument(
        "--nosym", dest="is_nosym", action="store_true",
        help="Symmetry is not imposed.")
    parser.add_argument(
        "--nothm", "--no-tetrahedron-method", dest="is_tetrahedron_method",
        action="store_false",
        help="Do not use tetrahedron method for DOS/PDOS")
    parser.add_argument(
        "--nowritemesh", dest="write_mesh", action="store_false",
        help="Do not write mesh.yaml or mesh.hdf5")
    parser.add_argument(
        "-p", "--plot", dest="is_graph_plot", action="store_true",
        help="Plot data")
    parser.add_argument(
        "--pa", "--primitive-axis", "--primitive-axes",
        nargs='+', dest="primitive_axes",
        help="Same as PRIMITIVE_AXES tag")
    parser.add_argument(
        "--pd", "--projection-direction", nargs='+',
        dest="projection_direction",
        help="Same as PROJECTION_DIRECTION tag")
    parser.add_argument(
        "--pdos", nargs='+', dest="pdos",
        help="Same as PDOS tag")
    parser.add_argument(
        "--pm", dest="is_plusminus_displacements", action="store_true",
        help="Set plus minus displacements")
    parser.add_argument(
        "--pr", "--pretend-real", dest="pretend_real", action="store_true",
        help=("Use imaginary frequency as real for thermal property "
              "calculation. For a testing purpose only, when a small "
              "amount of imaginary branches obtained."))
    parser.add_argument(
        "--pt", "--projected-thermal-property",
        dest="is_projected_thermal_properties", action="store_true",
        help="Output projected thermal properties")
    parser.add_argument(
        "--qe", "--pwscf", dest="qe_mode",
        action="store_true", help="Invoke Quantum espresso (QE) mode")
    parser.add_argument(
        "--qpoints", nargs='+', dest="qpoints",
        help="Calculate at specified q-points")
    parser.add_argument(
        "--qpoints-format", dest="qpoints_format",
        help="Q-points output file-format")
    parser.add_argument(
        "--q-direction", nargs='+', dest="nac_q_direction",
        help=("Direction of q-vector perturbation used for NAC at "
              "q->0, and group velocity for degenerate phonon "
              "mode in q-points mode"))
    parser.add_argument(
        "-q", "--quiet", dest="quiet", action="store_true",
        help="Print out smallest information")
    parser.add_argument(
        "--random-seed", dest="random_seed",
        type=int, help="Random seed by a 32 bit unsigned integer")
    parser.add_argument(
        "--rd", "--random-displacements", dest="random_displacements",
        type=int, help="Number of supercells with random displacements")
    parser.add_argument(
        "--readfc", dest="read_force_constants", action="store_true",
        help="Read FORCE_CONSTANTS")
    parser.add_argument(
        "--readfc-format", dest="readfc_format",
        help="Force constants input file-format")
    parser.add_argument(
        "--read-qpoints", dest="read_qpoints", action="store_true",
        help="Read QPOITNS")
    parser.add_argument(
        "-s", "--save", dest="is_graph_save", action="store_true",
        help="Save plot data in pdf")
    parser.add_argument(
        "--show-irreps", dest="show_irreps", action="store_true",
        help="Show IR-Reps along with characters")
    parser.add_argument(
        "--siesta", dest="siesta_mode", action="store_true",
        help="Invoke Siesta mode")
    parser.add_argument(
        "--sigma", dest="sigma",
        help="Smearing width for DOS")
    parser.add_argument(
        "--symmetry", dest="is_check_symmetry", action="store_true",
        help="Check crystal symmetry")
    parser.add_argument(
        "-t", "--thermal-property", dest="is_thermal_properties",
        action="store_true",
        help="Output thermal properties")
    parser.add_argument(
        "--td", "--thermal-displacements", dest="is_thermal_displacements",
        action="store_true",
        help="Output thermal displacements")
    parser.add_argument(
        "--tdm", "--thermal-displacement-matrix",
        dest="is_thermal_displacement_matrices", action="store_true",
        help="Output thermal displacement matrices")
    parser.add_argument(
        "--tdm-cif", "--thermal-displacement-matrix-cif",
        metavar='TEMPERATURE',
        dest="thermal_displacement_matrices_cif", type=float,
        help="Write cif with aniso_U for which temperature is specified")
    parser.add_argument(
        "--temperature", dest="temperature", type=float,
        metavar='TEMPERATURE', help="A temperature point")
    parser.add_argument(
        "--tmax", dest="tmax", type=float,
        help="Maximum calculated temperature")
    parser.add_argument(
        "--tmin", dest="tmin", type=float,
        help="Minimum calculated temperature")
    parser.add_argument(
        "--tolerance", dest="symmetry_tolerance", type=float,
        help="Symmetry tolerance to search")
    parser.add_argument(
        "--trigonal", dest="is_trigonal_displacements", action="store_true",
        help="Set displacements of all trigonal axes ")
    parser.add_argument(
        "--tstep", dest="tstep", type=float,
        help="Calculated temperature step")
    parser.add_argument(
        "--turbomole", dest="turbomole_mode", action="store_true",
        help="Invoke TURBOMOLE mode")
    parser.add_argument(
        "-v", "--verbose", dest="verbose", action="store_true",
        help="Detailed information is shown.")
    parser.add_argument(
        "--vasp", dest="vasp_mode", action="store_true",
        help="Invoke Vasp mode")
    parser.add_argument(
        "--wien2k", dest="wien2k_mode", action="store_true",
        help="Invoke Wien2k mode")
    parser.add_argument(
        "--wien2k_p1", dest="is_wien2k_p1", action="store_true",
        help="Assume Wien2k structs with displacements are P1")
    parser.add_argument(
        "--writefc", dest="write_force_constants", action="store_true",
        help="Write FORCE_CONSTANTS")
    parser.add_argument(
        "--writefc-format", dest="writefc_format",
        help="Force constants output file-format")
    parser.add_argument(
        "--writedm", dest="write_dynamical_matrices", action="store_true",
        help=("Write dynamical matrices. This has to be used "
              "with QPOINTS setting (or --qpoints)"))
    parser.add_argument(
        "--xyz-projection", dest="xyz_projection", action="store_true",
        help="Project PDOS x, y, z directions in Cartesian coordinates")
    parser.add_argument(
        "conf_file", nargs='*',
        help="Phonopy configure file")

    return parser, deprecated



# Copyright (C) 2015 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import numpy as np

from phonopy.structure.atoms import Atoms
from phonopy.units import dftbpToBohr
from phonopy.file_IO import collect_forces
from phonopy.interface.vasp import (get_scaled_positions_lines, check_forces,
                                    get_drift_forces)

def parse_set_of_forces(num_atoms, forces_filenames, verbose=True):
    hook = 'forces              :real:2:'
    is_parsed = True
    force_sets = []
    for i, filename in enumerate(forces_filenames):
        if verbose:
            sys.stdout.write("%d. " % (i + 1))

        f = open(filename)
        dftbp_forces = collect_forces(f, num_atoms, hook, [0, 1, 2])
        if check_forces(dftbp_forces, num_atoms, filename, verbose=verbose):
            drift_force = get_drift_forces(dftbp_forces,
                                           filename=filename,
                                           verbose=verbose)
            force_sets.append(np.array(dftbp_forces) - drift_force)
        else:
            is_parsed = False

    if is_parsed:
        return force_sets
    else:
        return []


#
# read dftbp-files
#

def read_dftbp(filename):
    """ Reads DFTB+ structure files in gen format.

    Args:
        filename: name of the gen-file to be read

    Returns:
        atoms: an object of the phonopy.Atoms class, representing the structure
        found in filename
    """

    infile = open(filename, 'r')

    lines = infile.readlines()

    # remove any comments
    for ss in lines:
        if ss.strip().startswith('#'):
            lines.remove(ss)

    natoms = int(lines[0].split()[0])
    symbols = lines[1].split()

    if (lines[0].split()[1].lower() == 'f'):
        is_scaled = True
        scale_pos = 1
        scale_latvecs = dftbpToBohr
    else:
        is_scaled = False
        scale_pos = dftbpToBohr
        scale_latvecs = dftbpToBohr

    # assign positions and expanded symbols
    positions = []
    expaned_symbols = []

    for ii in range(2, natoms+2):
        lsplit = lines[ii].split()

        expaned_symbols.append(symbols[int(lsplit[1]) - 1])
        positions.append([float(ss)*scale_pos for ss in lsplit[2:5]])

    # origin is ignored, may be used in future
    origin = [float(ss) for ss in lines[natoms+2].split()]

    # assign coords of unitcell
    cell = []

    for ii in range(natoms+3, natoms+6):
        lsplit = lines[ii].split()

        cell.append([float(ss)*scale_latvecs for ss in lsplit[:3]])
    cell = np.array(cell)

    if is_scaled:
        atoms = Atoms(symbols=expaned_symbols,
                      cell=cell,
                      scaled_positions=positions)
    else:
        atoms = Atoms(symbols=expaned_symbols,
                      cell=cell,
                      positions=positions)

    return atoms

#
# write dftb+ .gen-file
#
def get_reduced_symbols(symbols):
    """Reduces expanded list of symbols.

    Args:
        symbols: list containing any chemical symbols as often as
        the atom appears in the structure

    Returns:
        reduced_symbols: any symbols appears only once
    """

    reduced_symbols = []

    for ss in symbols:
        if not (ss in reduced_symbols):
            reduced_symbols.append(ss)

    return reduced_symbols

def write_dftbp(filename, atoms):
    """Writes DFTB+ readable, gen-formatted structure files

    Args:
        filename: name of the gen-file to be written
        atoms: object containing information about structure
    """
    scale_pos = dftbpToBohr

    lines = ""

    # 1. line, use absolute positions
    natoms = atoms.get_number_of_atoms()
    lines += str(natoms)
    lines += ' S \n'

    # 2. line
    expaned_symbols = atoms.get_chemical_symbols()
    symbols = get_reduced_symbols(expaned_symbols)
    lines += ' '.join(symbols) + '\n'

    atom_numbers = []
    for ss in expaned_symbols:
        atom_numbers.append(symbols.index(ss) + 1)

    positions = atoms.get_positions()/scale_pos

    for ii in range(natoms):
        pos = positions[ii]
        pos_str = "{:3d} {:3d} {:20.15f} {:20.15f} {:20.15f}\n".format(
            ii + 1, atom_numbers[ii], pos[0], pos[1], pos[2])
        lines += pos_str

    # origin arbitrary
    lines +='0.0 0.0 0.0\n'

    cell = atoms.get_cell()/scale_pos

    for ii in range(3):
        cell_str = "{:20.15f} {:20.15f} {:20.15f}\n".format(
            cell[ii][0], cell[ii][1], cell[ii][2])
        lines += cell_str

    outfile = open(filename, 'w')
    outfile.write(lines)

def write_supercells_with_displacements(supercell, cells_with_disps, filename="geo.gen"):
    """Writes perfect supercell and supercells with displacements

    Args:
        supercell: perfect supercell
        cells_with_disps: supercells with displaced atoms
        filename: root-filename
    """

    # original cell
    write_dftbp(filename + "S", supercell)

    # displaced cells
    for ii in range(len(cells_with_disps)):
        write_dftbp(filename + "S-{:03d}".format(ii+1), cells_with_disps[ii])
